/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2010 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  Jarre program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>

#include <alsa/asoundlib.h>

#include "jarre_midi_alsa_AlsaJNI.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    open_client
 * Signature: (Ljava/lang/String;Z)J
 */
JNIEXPORT jlong JNICALL Java_jarre_midi_alsa_AlsaJNI_open_1client (JNIEnv *env, jclass cls, jstring client_name, jboolean open_port)
{
	snd_seq_t *h;
    if (snd_seq_open(&h, "default", SND_SEQ_OPEN_DUPLEX, 0) < 0) return 0;
    const jbyte *str = (*env)->GetStringUTFChars(env, client_name, NULL);
    if (str == NULL) {
    	snd_seq_close(h);
    	return 0;
    }
    snd_seq_set_client_name(h, str);
    if (open_port) {
    	snd_seq_port_info_t *i;
    	snd_seq_port_info_malloc(&i);
    	snd_seq_port_info_set_port(i, 0);
    	snd_seq_port_info_set_name(i, str);
    	snd_seq_port_info_set_type(i,
    		SND_SEQ_PORT_TYPE_MIDI_GENERIC |
    		SND_SEQ_PORT_TYPE_APPLICATION
    	);
    	snd_seq_port_info_set_capability(i,
    		SND_SEQ_PORT_CAP_WRITE |
    		SND_SEQ_PORT_CAP_SUBS_WRITE |
    		SND_SEQ_PORT_CAP_READ |
    		SND_SEQ_PORT_CAP_SUBS_READ
    	);
    	snd_seq_create_port(h, i);
    }
	(*env)->ReleaseStringUTFChars(env, client_name, str);
	jlong handle = 0;
	memcpy(&handle, &h, sizeof(h));
	return handle;
}

/*
 * Class:     jarre_alsa_AlsaJNI
 * Method:    close_client
 * Signature: (J)V
 */
JNIEXPORT void JNICALL Java_jarre_midi_alsa_AlsaJNI_close_1client (JNIEnv *env, jclass cls, jlong handle)
{
	snd_seq_t *h = NULL;
	memcpy(&h, &handle, sizeof(h));
	snd_seq_close(h);
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    new_info
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_jarre_midi_alsa_AlsaJNI_new_1info (JNIEnv *env, jclass cls)
{
	snd_seq_client_info_t *info;
	snd_seq_client_info_malloc(&info);
	snd_seq_client_info_set_client(info, -1);
	jlong ptr = 0;
	memcpy(&ptr, &info, sizeof(info));
	return ptr;
}

/*
 * Class:     jarre_alsa_AlsaJNI
 * Method:    free_info
 * Signature: (J)V
 */
JNIEXPORT void JNICALL Java_jarre_midi_alsa_AlsaJNI_free_1info (JNIEnv *env, jclass cls, jlong info)
{
	snd_seq_client_info_t *i;
	memcpy(&i, &info, sizeof(i));
	snd_seq_client_info_free(i);
}

/*
 * Class:     jarre_alsa_AlsaJNI
 * Method:    query_next_client
 * Signature: (JJ)J
 */
JNIEXPORT jint JNICALL Java_jarre_midi_alsa_AlsaJNI_query_1next_1client (JNIEnv *env, jclass cls, jlong handle, jlong info)
{
	snd_seq_t *h;
	snd_seq_client_info_t *i;
	memcpy(&h, &handle, sizeof(h));
	memcpy(&i, &info, sizeof(i));
	return snd_seq_query_next_client(h, i);
}

/*
 * Class:     jarre_alsa_AlsaJNI
 * Method:    info_get_name
 * Signature: (J)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_jarre_midi_alsa_AlsaJNI_info_1get_1name (JNIEnv *env, jclass cls, jlong info)
{
	snd_seq_client_info_t *i;
	memcpy(&i, &info, sizeof(i));
	jstring str = (*env)->NewStringUTF(env, snd_seq_client_info_get_name(i));
	return str;
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    info_get_id
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_jarre_midi_alsa_AlsaJNI_info_1get_1id (JNIEnv *env, jclass cls, jlong info)
{
	snd_seq_client_info_t *i;
	memcpy(&i, &info, sizeof(i));
	return snd_seq_client_info_get_client(i);
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    connect_port
 * Signature: (JIII)V
 */
JNIEXPORT void JNICALL Java_jarre_midi_alsa_AlsaJNI_connect_1port_1to (JNIEnv *env, jclass cls, jlong handle, jint port, jint dest_client, jint dest_port)
{
	snd_seq_t *h;
	memcpy(&h, &handle, sizeof(h));
	snd_seq_connect_to(h, port, dest_client, dest_port);
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    disconnect_port
 * Signature: (JIII)V
 */
JNIEXPORT void JNICALL Java_jarre_midi_alsa_AlsaJNI_disconnect_1port_1to (JNIEnv *env, jclass cls, jlong handle, jint port, jint dest_client, jint dest_port)
{
	snd_seq_t *h;
	memcpy(&h, &handle, sizeof(h));
	snd_seq_disconnect_to(h, port, dest_client, dest_port);
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    connect_port
 * Signature: (JIII)V
 */
JNIEXPORT void JNICALL Java_jarre_midi_alsa_AlsaJNI_connect_1port_1from (JNIEnv *env, jclass cls, jlong handle, jint port, jint src_client, jint src_port)
{
	snd_seq_t *h;
	memcpy(&h, &handle, sizeof(h));
	snd_seq_connect_from(h, port, src_client, src_port);
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    disconnect_port
 * Signature: (JIII)V
 */
JNIEXPORT void JNICALL Java_jarre_midi_alsa_AlsaJNI_disconnect_1port_1from (JNIEnv *env, jclass cls, jlong handle, jint port, jint src_client, jint src_port)
{
	snd_seq_t *h;
	memcpy(&h, &handle, sizeof(h));
	snd_seq_disconnect_from(h, port, src_client, src_port);
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    new_port_info
 * Signature: (J)J
 */
JNIEXPORT jlong JNICALL Java_jarre_midi_alsa_AlsaJNI_new_1port_1info (JNIEnv *env, jclass cls, jlong info)
{
	snd_seq_client_info_t *i = NULL;
	memcpy(&i, &info, sizeof(i));
	snd_seq_port_info_t *pi;
	snd_seq_port_info_malloc(&pi);
	snd_seq_port_info_set_port(pi, -1);
	snd_seq_port_info_set_client(pi, snd_seq_client_info_get_client(i));
	jlong port_info = 0;
	memcpy(&port_info, &pi, sizeof(pi));
	return port_info;
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    free_port_info
 * Signature: (J)J
 */
JNIEXPORT void JNICALL Java_jarre_midi_alsa_AlsaJNI_free_1port_1info (JNIEnv *env, jclass cls, jlong port_info)
{
	snd_seq_port_info_t *pi;
	memcpy(&pi, &port_info, sizeof(pi));
	snd_seq_port_info_free(pi);
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    query_next_port
 * Signature: (JJ)I
 */
JNIEXPORT jint JNICALL Java_jarre_midi_alsa_AlsaJNI_query_1next_1port (JNIEnv *env, jclass cls, jlong handle, jlong port_info)
{
	snd_seq_t *h;
	snd_seq_port_info_t *pi;
	memcpy(&h, &handle, sizeof(h));
	memcpy(&pi, &port_info, sizeof(pi));
	return snd_seq_query_next_port(h, pi);
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    port_get_name
 * Signature: (J)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_jarre_midi_alsa_AlsaJNI_port_1get_1name (JNIEnv *env, jclass cls, jlong port_info)
{
	snd_seq_port_info_t *pi;
	memcpy(&pi, &port_info, sizeof(pi));
	jstring str = (*env)->NewStringUTF(env, snd_seq_port_info_get_name(pi));
	return str;
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    port_can_write
 * Signature: (J)Z
 */
JNIEXPORT jboolean JNICALL Java_jarre_midi_alsa_AlsaJNI_port_1can_1write (JNIEnv *env, jclass cls, jlong port_info)
{
	snd_seq_port_info_t *pi;
	memcpy(&pi, &port_info, sizeof(pi));
	unsigned int bits = SND_SEQ_PORT_CAP_WRITE | SND_SEQ_PORT_CAP_SUBS_WRITE | SND_SEQ_PORT_TYPE_MIDI_GENERIC;
	return (snd_seq_port_info_get_capability(pi) & bits) == bits;
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    port_can_read
 * Signature: (J)Z
 */
JNIEXPORT jboolean JNICALL Java_jarre_midi_alsa_AlsaJNI_port_1can_1read (JNIEnv *env, jclass cls, jlong port_info)
{
	snd_seq_port_info_t *pi;
	memcpy(&pi, &port_info, sizeof(pi));
	unsigned int bits = SND_SEQ_PORT_CAP_READ | SND_SEQ_PORT_CAP_SUBS_READ | SND_SEQ_PORT_TYPE_MIDI_GENERIC;
	return (snd_seq_port_info_get_capability(pi) & bits) == bits;
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    port_get_id
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_jarre_midi_alsa_AlsaJNI_port_1get_1id (JNIEnv *env, jclass cls, jlong port_info)
{
	snd_seq_port_info_t *pi;
	memcpy(&pi, &port_info, sizeof(pi));
	return snd_seq_port_info_get_port(pi);
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    send_note_on
 * Signature: (JIIIII)V
 */
JNIEXPORT void JNICALL Java_jarre_midi_alsa_AlsaJNI_send_1note_1on (JNIEnv *env, jclass cls, jlong handle, jint dest_addr, jint dest_port, jint channel, jint note, jint velocity)
{
	snd_seq_t *h;
	memcpy(&h, &handle, sizeof(h));
	snd_seq_event_t e;
	snd_seq_ev_clear(&e);
	snd_seq_ev_set_direct(&e);
	snd_seq_ev_set_source(&e, 0);
	snd_seq_ev_set_dest(&e, dest_addr, dest_port);
	snd_seq_ev_set_noteon(&e, channel, note, velocity);
	if (snd_seq_event_output_direct(h, &e) < 0) {
		return;
	}
	snd_seq_drain_output(h);
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    send_note_off
 * Signature: (JIIIII)V
 */
JNIEXPORT void JNICALL Java_jarre_midi_alsa_AlsaJNI_send_1note_1off (JNIEnv *env, jclass cls, jlong handle, jint dest_addr, jint dest_port, jint channel, jint note, jint velocity)
{
	snd_seq_t *h;
	memcpy(&h, &handle, sizeof(h));
	snd_seq_event_t e;
	snd_seq_ev_clear(&e);
	snd_seq_ev_set_direct(&e);
	snd_seq_ev_set_source(&e, 0);
	snd_seq_ev_set_dest(&e, dest_addr, dest_port);
	snd_seq_ev_set_noteoff(&e, channel, note, velocity);
	if (snd_seq_event_output_direct(h, &e) < 0) {
		return;
	}
	snd_seq_drain_output(h);
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    send_program_change
 * Signature: (JIIII)V
 */
JNIEXPORT void JNICALL Java_jarre_midi_alsa_AlsaJNI_send_1program_1change (JNIEnv *env, jclass cls, jlong handle, jint dest_addr, jint dest_port, jint channel, jint value)
{
	snd_seq_t *h;
	memcpy(&h, &handle, sizeof(h));
	snd_seq_event_t e;
	snd_seq_ev_clear(&e);
	snd_seq_ev_set_direct(&e);
	snd_seq_ev_set_source(&e, 0);
	snd_seq_ev_set_dest(&e, dest_addr, dest_port);
	snd_seq_ev_set_pgmchange(&e, channel, value);
	if (snd_seq_event_output_direct(h, &e) < 0) {
		return;
	}
	snd_seq_drain_output(h);
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    send_controller
 * Signature: (JIIIII)V
 */
JNIEXPORT void JNICALL Java_jarre_midi_alsa_AlsaJNI_send_1controller (JNIEnv *env, jclass cls, jlong handle, jint dest_addr, jint dest_port, jint channel, jint param, jint value)
{
	snd_seq_t *h;
	memcpy(&h, &handle, sizeof(h));
	snd_seq_event_t e;
	snd_seq_ev_clear(&e);
	snd_seq_ev_set_direct(&e);
	snd_seq_ev_set_source(&e, 0);
	snd_seq_ev_set_dest(&e, dest_addr, dest_port);
	snd_seq_ev_set_controller(&e, channel, param, value);
	if (snd_seq_event_output_direct(h, &e) < 0) {
		return;
	}
	snd_seq_drain_output(h);

}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    send_pitchbend
 * Signature: (JIIII)V
 */
JNIEXPORT void JNICALL Java_jarre_midi_alsa_AlsaJNI_send_1pitchbend (JNIEnv *env, jclass cls, jlong handle, jint dest_addr, jint dest_port, jint channel, jint value)
{
	snd_seq_t *h;
	memcpy(&h, &handle, sizeof(h));
	snd_seq_event_t e;
	snd_seq_ev_clear(&e);
	snd_seq_ev_set_direct(&e);
	snd_seq_ev_set_source(&e, 0);
	snd_seq_ev_set_dest(&e, dest_addr, dest_port);
	snd_seq_ev_set_pitchbend(&e, channel, value);
	if (snd_seq_event_output_direct(h, &e) < 0) {
		return;
	}
	snd_seq_drain_output(h);
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    send_chanpress
 * Signature: (JIIII)V
 */
JNIEXPORT void JNICALL Java_jarre_midi_alsa_AlsaJNI_send_1chanpress (JNIEnv *env, jclass cls, jlong handle, jint dest_addr, jint dest_port, jint channel, jint value)
{
	snd_seq_t *h;
	memcpy(&h, &handle, sizeof(h));
	snd_seq_event_t e;
	snd_seq_ev_clear(&e);
	snd_seq_ev_set_direct(&e);
	snd_seq_ev_set_source(&e, 0);
	snd_seq_ev_set_dest(&e, dest_addr, dest_port);
	snd_seq_ev_set_chanpress(&e, channel, value);
	if (snd_seq_event_output_direct(h, &e) < 0) {
		return;
	}
	snd_seq_drain_output(h);
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    event_pending
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_jarre_midi_alsa_AlsaJNI_event_1pending (JNIEnv *env, jclass cls, jlong handle)
{
	snd_seq_t *h;
	memcpy(&h, &handle, sizeof(h));
	return snd_seq_event_input_pending(h, 1);
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    retrieve_event
 * Signature: (J)J
 */
JNIEXPORT jlong JNICALL Java_jarre_midi_alsa_AlsaJNI_retrieve_1event (JNIEnv *env, jclass cls, jlong handle)
{
	snd_seq_t *h;
	memcpy(&h, &handle, sizeof(h));
	snd_seq_event_t *e;
	snd_seq_event_input(h, &e);
	jlong event = 0;
	memcpy(&event, &e, sizeof(e));
	return event;
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    free_event
 * Signature: (J)V
 */
JNIEXPORT void JNICALL Java_jarre_midi_alsa_AlsaJNI_free_1event (JNIEnv *env, jclass cls, jlong event)
{
	snd_seq_event_t *e;
	memcpy(&e, &event, sizeof(e));
	snd_seq_free_event(e);
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    event_get_type
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_jarre_midi_alsa_AlsaJNI_event_1get_1type (JNIEnv *env, jclass cls, jlong event)
{
	snd_seq_event_t *e;
	memcpy(&e, &event, sizeof(e));
	return e->type;
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    event_get_channel
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_jarre_midi_alsa_AlsaJNI_event_1get_1channel (JNIEnv *env, jclass cls, jlong event)
{
	snd_seq_event_t *e;
	memcpy(&e, &event, sizeof(e));
	switch (e->type) {
	    case SND_SEQ_EVENT_NOTE:
	    case SND_SEQ_EVENT_NOTEON:
	    case SND_SEQ_EVENT_NOTEOFF:
	    case SND_SEQ_EVENT_KEYPRESS:
	    case SND_SEQ_EVENT_CONTROLLER:
	    case SND_SEQ_EVENT_PGMCHANGE:
	    case SND_SEQ_EVENT_CHANPRESS:
	    case SND_SEQ_EVENT_PITCHBEND:
	    case SND_SEQ_EVENT_CONTROL14:
	    case SND_SEQ_EVENT_NONREGPARAM:
	    case SND_SEQ_EVENT_REGPARAM:
	    case SND_SEQ_EVENT_SONGPOS:
	    case SND_SEQ_EVENT_SONGSEL:
	    case SND_SEQ_EVENT_QFRAME:
	    case SND_SEQ_EVENT_TIMESIGN:
	    case SND_SEQ_EVENT_KEYSIGN:
	    	return e->data.note.channel;
	}
	return -1;
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    event_get_data1
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_jarre_midi_alsa_AlsaJNI_event_1get_1data1 (JNIEnv *env, jclass cls, jlong event)
{
	snd_seq_event_t *e;
	memcpy(&e, &event, sizeof(e));
	switch (e->type) {
		case SND_SEQ_EVENT_NOTE:
		case SND_SEQ_EVENT_NOTEON:
		case SND_SEQ_EVENT_NOTEOFF:
		case SND_SEQ_EVENT_KEYPRESS:
			return e->data.note.note;
		case SND_SEQ_EVENT_CONTROLLER:
		case SND_SEQ_EVENT_PGMCHANGE:
		case SND_SEQ_EVENT_CHANPRESS:
		case SND_SEQ_EVENT_PITCHBEND:
		case SND_SEQ_EVENT_CONTROL14:
		case SND_SEQ_EVENT_NONREGPARAM:
		case SND_SEQ_EVENT_REGPARAM:
		case SND_SEQ_EVENT_SONGPOS:
		case SND_SEQ_EVENT_SONGSEL:
		case SND_SEQ_EVENT_QFRAME:
		case SND_SEQ_EVENT_TIMESIGN:
		case SND_SEQ_EVENT_KEYSIGN:
			return e->data.control.param;
	}
	return 0;
}

/*
 * Class:     jarre_midi_alsa_AlsaJNI
 * Method:    event_get_data2
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_jarre_midi_alsa_AlsaJNI_event_1get_1data2 (JNIEnv *env, jclass cls, jlong event)
{
	snd_seq_event_t *e;
	memcpy(&e, &event, sizeof(e));
	switch (e->type) {
		case SND_SEQ_EVENT_NOTE:
		case SND_SEQ_EVENT_NOTEON:
		case SND_SEQ_EVENT_NOTEOFF:
		case SND_SEQ_EVENT_KEYPRESS:
			return e->data.note.velocity;
		case SND_SEQ_EVENT_CONTROLLER:
		case SND_SEQ_EVENT_PGMCHANGE:
		case SND_SEQ_EVENT_CHANPRESS:
		case SND_SEQ_EVENT_PITCHBEND:
		case SND_SEQ_EVENT_CONTROL14:
		case SND_SEQ_EVENT_NONREGPARAM:
		case SND_SEQ_EVENT_REGPARAM:
		case SND_SEQ_EVENT_SONGPOS:
		case SND_SEQ_EVENT_SONGSEL:
		case SND_SEQ_EVENT_QFRAME:
		case SND_SEQ_EVENT_TIMESIGN:
		case SND_SEQ_EVENT_KEYSIGN:
			return e->data.control.value;
	}
	return 0;
}

#ifdef __cplusplus
}
#endif
