/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2010 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  Jarre program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gtk/gtk.h>

#include "jarre_widgets_internal_GtkJNI.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Class:     jarre_widgets_internal_GtkJNI
 * Method:    widget_get_default_style
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_jarre_widgets_internal_GtkJNI_widget_1get_1default_1style (JNIEnv *env , jclass cls)
{
	GtkStyle *s = gtk_widget_get_default_style();
	jlong style = 0;
	memcpy(&style, &s, sizeof(s));
	return style;
}

/*
 * Class:     jarre_widgets_internal_GtkJNI
 * Method:    icon_factory_lookup_default
 * Signature: (Ljava/lang/String;)J
 */
JNIEXPORT jlong JNICALL Java_jarre_widgets_internal_GtkJNI_icon_1factory_1lookup_1default (JNIEnv *env, jclass cls, jstring name)
{
	const jbyte *str = (*env)->GetStringUTFChars(env, name, NULL);
	GtkIconSet *i = gtk_icon_factory_lookup_default(str);
	(*env)->ReleaseStringUTFChars(env, name, str);
	jlong icon_set = 0;
	memcpy(&icon_set, &i, sizeof(i));
	return icon_set;
}

/*
 * Class:     jarre_widgets_internal_GtkJNI
 * Method:    icon_set_render_icon
 * Signature: (JJIIIJLjava/lang/String;)J
 */
JNIEXPORT jlong JNICALL Java_jarre_widgets_internal_GtkJNI_icon_1set_1render_1icon (JNIEnv *env , jclass cls, jlong icon_set, jlong style, jint direction, jint state, jint size, jlong widget, jstring details)
{
	GtkIconSet *i = NULL;
	GtkStyle *s = NULL;
	GtkWidget *w = NULL;
	memcpy(&i, &icon_set, sizeof(i));
	memcpy(&s, &style, sizeof(s));
	memcpy(&w, &widget, sizeof(w));
	const jbyte *str = NULL;
	if (details != NULL) str = (*env)->GetStringUTFChars(env, details, NULL);
	GdkPixbuf *p = gtk_icon_set_render_icon(i, s, direction, state, size, w, str);
	if (details != NULL) (*env)->ReleaseStringUTFChars(env, details, str);
	jlong pixbuf = 0;
	memcpy(&pixbuf, &p, sizeof(p));
	return pixbuf;
}

/*
 * Class:     jarre_widgets_internal_GtkJNI
 * Method:    pixbuf_get_width
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_jarre_widgets_internal_GtkJNI_pixbuf_1get_1width (JNIEnv *env, jclass cls, jlong pixbuf)
{
	GdkPixbuf *p = NULL;
	memcpy(&p, &pixbuf, sizeof(p));
	return gdk_pixbuf_get_width(p);
}

/*
 * Class:     jarre_widgets_internal_GtkJNI
 * Method:    pixbuf_get_height
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_jarre_widgets_internal_GtkJNI_pixbuf_1get_1height (JNIEnv *env, jclass cls, jlong pixbuf)
{
	GdkPixbuf *p = NULL;
	memcpy(&p, &pixbuf, sizeof(p));
	return gdk_pixbuf_get_height(p);
}

/*
 * Class:     jarre_widgets_internal_GtkJNI
 * Method:    pixbuf_get_rowstride
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_jarre_widgets_internal_GtkJNI_pixbuf_1get_1rowstride (JNIEnv *env, jclass cls, jlong pixbuf)
{
	GdkPixbuf *p = NULL;
	memcpy(&p, &pixbuf, sizeof(p));
	return gdk_pixbuf_get_rowstride(p);
}

/*
 * Class:     jarre_widgets_internal_GtkJNI
 * Method:    pixbuf_get_has_alpha
 * Signature: (J)Z
 */
JNIEXPORT jboolean JNICALL Java_jarre_widgets_internal_GtkJNI_pixbuf_1get_1has_1alpha (JNIEnv *env, jclass cls, jlong pixbuf)
{
	GdkPixbuf *p = NULL;
	memcpy(&p, &pixbuf, sizeof(p));
	return gdk_pixbuf_get_has_alpha(p);
}

/*
 * Class:     jarre_widgets_internal_GtkJNI
 * Method:    pixbuf_get_pixels
 * Signature: (J)[B
 */
JNIEXPORT jbyteArray JNICALL Java_jarre_widgets_internal_GtkJNI_pixbuf_1get_1pixels (JNIEnv *env, jclass cls, jlong pixbuf)
{
	GdkPixbuf *p = NULL;
	memcpy(&p, &pixbuf, sizeof(p));
	int height = gdk_pixbuf_get_height (p);
	int rowstride = gdk_pixbuf_get_rowstride (p);
	int size = height * rowstride;
	guchar* data = gdk_pixbuf_get_pixels(p);
	jbyteArray jarray = (*env)->NewByteArray(env, size);
	(*env)->SetByteArrayRegion(env, jarray, 0, size, (jbyte*)data);
	return jarray;
}

/*
 * Class:     jarre_widgets_internal_GtkJNI
 * Method:    object_unref
 * Signature: (J)V
 */
JNIEXPORT void JNICALL Java_jarre_widgets_internal_GtkJNI_object_1unref (JNIEnv *env, jclass cls, jlong pixbuf)
{
	GdkPixbuf *p = NULL;
	memcpy(&p, &pixbuf, sizeof(p));
	g_object_unref(p);
}

#ifdef __cplusplus
}
#endif
