/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.app;

import static jarre.i18n.I18n._;
import jarre.Config;
import jarre.midi.MidiDriver;
import jarre.widgets.windows.AppWindow;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import javax.sound.midi.MidiDevice;
import javax.sound.midi.MidiDevice.Info;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;

public class ConfigWindow extends AppWindow {
    public ConfigWindow(Shell shell, String title) {
	super(new Shell(shell, SWT.APPLICATION_MODAL | SWT.BORDER |  SWT.CLOSE | SWT.DIALOG_TRIM), title);
	getShell().pack();
    }
    @Override public void createControl(Composite parent) {
	String inName = Config.xmlConfig.getString("config.device.in", null);
	String outName = Config.xmlConfig.getString("config.device.out", null);
	String driverName = Config.xmlConfig.getString("config.device.driver", MidiDriver.MIDI_JAVA);
	parent.setLayout(new GridLayout(2, false));
	new Label(parent, SWT.NONE).setText(_("Midi driver: "));
	combo = new Combo(parent, SWT.READ_ONLY | SWT.BORDER | SWT.FLAT | SWT.NO_FOCUS);
	combo.add("Java");
	if (! SWT.getPlatform().equals("win32"))
	    combo.add("Alsa");
	if (driverName.equals(MidiDriver.MIDI_ALSA))
	    combo.select(1);
	else combo.select(0);
	combo.addListener(SWT.Selection, new Listener() {
	    @Override public void handleEvent(Event event) {
		inMenu.dispose();
		outMenu.dispose();
		inButton.setText(_("Choose a device"));
		outButton.setText(_("Choose a device"));
		getShell().layout(true, true);
		getShell().pack();
		String driverName = getDeviceName(combo.getText());
		inMenu = createMenu(inButton, MidiDriver.getInDevices(driverName));
		outMenu = createMenu(outButton, MidiDriver.getOutDevices(driverName));
		saveSettings();
	    }	    
	});
	new Label(parent, SWT.NONE).setText(_("MIDI input device: "));
	inButton = new Button(parent, SWT.PUSH);
	if (inName == null) inButton.setText(_("Choose a device"));
	else inButton.setText(inName);
	inButton.addListener(SWT.Selection, new Listener() {
	    @Override public void handleEvent(Event event) {
		Point pos = getDisplay().map(inButton, null, event.x, event.y);
		inMenu.setLocation(pos);
		inMenu.setVisible(true);
	    }	    
	});
	inMenu = createMenu(inButton, MidiDriver.getInDevices(driverName));
	new Label(parent, SWT.NONE).setText(_("MIDI output device: "));
	outButton = new Button(parent, SWT.PUSH);
	if (outName == null) outButton.setText(_("Choose a device"));
	else outButton.setText(outName);
	outButton.addListener(SWT.Selection, new Listener() {
	    @Override public void handleEvent(Event event) {
		Point pos = getDisplay().map(outButton, null, event.x, event.y);
		outMenu.setLocation(pos);
		outMenu.setVisible(true);
	    }	    
	});
	outMenu = createMenu(outButton, MidiDriver.getOutDevices(driverName));
	Button button = new Button(parent, SWT.CHECK);
	button.setText(_("Connect input to output."));
	button.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, true, 2, 1));
	button.setSelection(Config.xmlConfig.getBoolean("config.device[@redirect]", false));
	button.addListener(SWT.Selection, new Listener() {
	    @Override public void handleEvent(Event event) {
		Button button = (Button)event.widget;
		Config.xmlConfig.setProperty("config.device[@redirect]", button.getSelection());
	    } 
	});
	new Label(parent, SWT.NONE);
	button = new Button(parent, SWT.PUSH);
	button.setText("     " + "OK" + "     ");
	button.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
	button.addListener(SWT.Selection, new Listener() {
	    @Override public void handleEvent(Event event) {
		saveSettings();
		close(SWT.OK);
	    } 
	});
    }
    private void saveSettings() {
	String inText = inButton.getText();
	String outText = outButton.getText();
	if (inText.equals(_("Choose a device"))) inText = "";
	if (outText.equals(_("Choose a device"))) outText = "";
	Config.xmlConfig.setProperty("config.device.in", inText);
	Config.xmlConfig.setProperty("config.device.out", outText);
	Config.xmlConfig.setProperty("config.device.driver", getDeviceName(combo.getText()));	
    }
    private String getDeviceName(String text) {
	if (text.equalsIgnoreCase("java")) return MidiDriver.MIDI_JAVA;
	else if (text.equalsIgnoreCase("alsa")) return MidiDriver.MIDI_ALSA;
	else return null;
    }
    private Menu createMenu(Button button, List<MidiDevice> devices) {
	final HashMap<String, LinkedList<Info>> infoMap = new HashMap<String, LinkedList<Info>>(); 
	for (MidiDevice d: devices) {
	    Info info = d.getDeviceInfo();
	    String desc = info.getDescription();
	    LinkedList<Info> infoList = infoMap.get(desc);
	    if (infoList == null) {
		infoList = new LinkedList<Info>();
		infoMap.put(desc, infoList);
	    }
	    infoList.add(info);
	}
	Menu menu = new Menu(getShell(), SWT.POP_UP);
	if (infoMap.size() == 1) createMenuItems(button, menu, infoMap.values().iterator().next());
	else {
	    for (String s: infoMap.keySet()) {
		MenuItem item = new MenuItem(menu, SWT.CASCADE);
		item.setText(s);
		Menu subMenu = new Menu(getShell(), SWT.DROP_DOWN);
		item.setMenu(subMenu);
		createMenuItems(button, subMenu, infoMap.get(s));
	    }
	}
	return menu;
    }
    private void createMenuItems(final Button button, Menu menu, LinkedList<Info> infoList) {
	for (Info i: infoList) {
	    MenuItem item = new MenuItem(menu, SWT.PUSH);
	    final String name = i.getName();
	    item.setText(name);
	    item.addListener(SWT.Selection, new Listener() {
		@Override public void handleEvent(Event event) {
		    button.setText(name);
		    button.pack(true);
		    button.redraw();
		    button.update();
		    getShell().pack(true);
		    getShell().redraw();
		    getShell().update();
		}
	    });
	}
    }
    private Combo combo;
    private Button inButton;
    private Button outButton;
    private Menu inMenu;
    private Menu outMenu;
}
