/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.app.widgets;

import static jarre.i18n.I18n._;
import jarre.Jarre;
import jarre.midi.MySequence;
import jarre.widgets.PlayerButtons.ActionListener;
import jarre.widgets.utils.GridLayoutFactory;
import jarre.widgets.utils.Utils;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.LinkedList;
import java.util.List;

import javax.sound.midi.InvalidMidiDataException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;

public class Controls extends Composite {
    private OpenFileWidget openFile;
    private PlayerWidget playerWidget;
    private TracksWidget tracksWidget;
    private Button metronomeButton;
    private Button followButton;
    private Button shrinkButton;
    private Button ignoreButton;
    private List<SequenceChange> changeList = new LinkedList<SequenceChange>();
    public Controls(Composite parent, int style) {
	super(parent, style);
	GridLayoutFactory.swtDefaults().applyTo(this);
	openFile = new OpenFileWidget(this, SWT.NONE);
	openFile.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
	openFile.addListener(SWT.Selection, new Listener() {
	    @Override public void handleEvent(Event event) {
		try {
		    final String fileName = openFile.getFileName();
		    if (fileName != null) {
			final InputStream file = new FileInputStream(fileName);
			Jarre.window.execTask(new Runnable() {
			    @Override public void run() {
				getDisplay().syncExec(new Runnable() {
				    @Override public void run() {
					Jarre.window.pause();
				    }
				});
				try {
				    final MySequence sequence = MySequence.create(file);
				    getDisplay().syncExec(new Runnable() {
					@Override public void run() {
					    tracksWidget.setSequence(sequence);
					    playerWidget.setSongTime((int)(sequence.getMicrosecondLength() / 1000000));
					    for (SequenceChange i: changeList) i.hanldeEvent(sequence);
					}
				    });
				} catch (InvalidMidiDataException e) {
				    getDisplay().syncExec(new Runnable() {
					@Override public void run() {
					    MessageBox box = new MessageBox(getShell(), SWT.ICON_ERROR | SWT.OK);
					    box.setMessage(_("Cannot open midi file. Corrupted data.") + "\n\n" + fileName);
					    box.open();
					}
				    });
				}
			    }
			});
		    }
		} catch (FileNotFoundException e) {
		}
	    }
	});
	openFile.setToolTipText(_("Shortcut: o"));
	playerWidget = new PlayerWidget(this, SWT.NONE, new String[] {
	    _("Shortcut: p"), _("Shortcut: b"), _("Shortcut: arrow"), _("Shortcut: arrow")
	});
	playerWidget.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
	tracksWidget = new TracksWidget(this, SWT.BORDER);
	tracksWidget.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
	Utils.disableFocus(tracksWidget);
	metronomeButton = new Button(this, SWT.CHECK);
	metronomeButton.setText(_("Turn on metronome"));
	metronomeButton.setToolTipText(_("Shortcut: t"));
	Utils.disableFocus(metronomeButton);
	followButton = new Button(this, SWT.CHECK);
	followButton.setText(_("\"Follow you\" mode"));
	followButton.setToolTipText(_("Shortcut: m"));
	Utils.disableFocus(followButton);
	ignoreButton = new Button(this, SWT.CHECK);
	ignoreButton.setText(_("Ignore program change"));
	ignoreButton.setToolTipText(_("Shortcut: i"));
	Utils.disableFocus(ignoreButton);
	shrinkButton = new Button(this, SWT.CHECK);
	shrinkButton.setText(_("Shrink keyboard"));
	shrinkButton.setToolTipText(_("Shortcut: s"));
	Utils.disableFocus(shrinkButton);
    }
    public void addProgressBarListener(Listener listener) {
	playerWidget.addProgressBarListener(listener);
    }
    public void addActionListener(ActionListener listener) {
	playerWidget.addActionListener(listener);
    }
    public void addSequenceListener(SequenceChange i) {
	changeList.add(i);
    }
    public void addMetronomeListener(Listener listener) {
	metronomeButton.addListener(SWT.Selection, listener);
    }
    public void addFollowListener(Listener listener) {
	followButton.addListener(SWT.Selection, listener);
    }
    public void addShrinkListener(Listener listener) {
	shrinkButton.addListener(SWT.Selection, listener);
    }
    public void addIgnoreListener(Listener listener) {
	ignoreButton.addListener(SWT.Selection, listener);
    }
    public void addTrackChangeListener(Listener listener) {
	tracksWidget.addTrackChangeListener(listener);
    }
    public PlayerWidget getPlayerWidget() {
	return playerWidget;
    }
    public Button getMetronomeButton() {
	return metronomeButton;
    }
    public Button getFollowButton() {
	return followButton;
    }
    public Button getShrinkButton() {
	return shrinkButton;
    }
    public Button getIgnoreButton() {
	return ignoreButton;
    }
    public void openFileDialog() {
	openFile.openFileDialog();
    }
    public static interface SequenceChange {
	void hanldeEvent(MySequence sequence);
    }
}
