/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.app.widgets;

import static jarre.i18n.I18n._;
import jarre.Config;
import jarre.widgets.utils.CustomSize;
import jarre.widgets.utils.Utils;

import java.io.File;
import java.util.ArrayList;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

public class OpenFileWidget extends Composite {
    private Combo combo;
    private Button button;
    private String fileName;
    private ArrayList<String> list = new ArrayList<String>(21);
    public OpenFileWidget(Composite parent, int style) {
	super(parent, style);
	setLayout(new GridLayout(2, false));
	CustomSize cs = new CustomSize(this, SWT.NONE);
	combo = new Combo(cs, SWT.READ_ONLY | SWT.BORDER | SWT.FLAT | SWT.NO_FOCUS);
	combo.setLayoutData(new GridData(SWT.BEGINNING, SWT.FILL, false, false));
	combo.setText("                    ");
	combo.addListener(SWT.Selection, new Listener() {
	    @Override public void handleEvent(Event event) {
		int index = combo.getSelectionIndex();
		if (index >= 0) {
		    fileName = list.get(index);
		    OpenFileWidget.this.notifyListeners(SWT.Selection, event);
		}
	    }
	});
	Utils.disableFocus(combo);
	cs.setControl(combo);
	cs.setWidth(180);
	String[] array = Config.getPropertyArray("file.history");
	for (String s: array) {
	    File file = new File(s);
	    if (file.exists()) {
		String[] x = s.split("\\" + File.separator);
		combo.add(x[x.length - 1]);
		list.add(s);
	    }
	}
	button = new Button(this, SWT.PUSH);
	button.setImage(new Image(getDisplay(), "data/document-open.png"));
	button.addListener(SWT.Selection, new Listener() {
	    @Override public void handleEvent(Event event) {
		openFileDialog();
	    }
	});
	Utils.disableFocus(button);
	button.setToolTipText(_("Shortcut: o"));
    }
    public void openFileDialog() {
	DirectoryDialog dialog = new DirectoryDialog(getShell(), SWT.OPEN);
	dialog.setFileFilter(new String[] { "^.*\\.[Mm][Ii][Dd]$", "^.*\\.[Mm][Ii][Dd][Ii]$" });
	dialog.open();
	File file = dialog.getSelection();
	if (file != null) {
	    fileName = file.getAbsolutePath();
	    boolean exists = false;
	    for (String s: list)
		if (s.equalsIgnoreCase(fileName)) {
		    exists = true;
		    break;
		}
	    if (! exists) {
		combo.add(file.getName(), 0);
		combo.select(0);
		list.add(0, fileName);
		int count = combo.getItemCount();
		if (count > 20) {
		    combo.remove(20);
		    list.remove(20);
		}
		Config.setStringArray("file.history", list.toArray(new String[0]));
		OpenFileWidget.this.notifyListeners(SWT.Selection, new Event());
	    }
	}
    }
    public String getFileName() {
	return fileName;
    }
}
