/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.app.widgets;

import jarre.Config;
import jarre.keyboard.KeyboardPainterManager;
import jarre.midi.MySequence;
import jarre.widgets.common.BasicControl;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

public abstract class PianoAbstract extends BasicControl {
    private static RGB[] trackColors;
    private static RGB defaultColor;
    static {
	defaultColor = new RGB(
		Config.xmlConfig.getInt("InputColor.red", 30),
		Config.xmlConfig.getInt("InputColor.green", 144),
		Config.xmlConfig.getInt("InputColor.blue", 255));
	trackColors = new RGB[256];
	for (int i = 0; i < trackColors.length; i++) trackColors[i] = defaultColor;
    }
    public PianoAbstract(Composite parent, int style) {
	super(parent, style);
	addListener(SWT.Resize, new Listener() {
	    private int oldWidth = -1;
	    @Override public void handleEvent(Event event) {
		Point size = getSize();
		if (size.x != oldWidth) {
		    oldWidth = size.x;
		    KeyboardPainterManager.setKeyboardSize(size.x, size.y);
		}
	    }
	});
    }
    public static void shrinkKeyboard(MySequence sequence) {
	if (sequence == null) KeyboardPainterManager.setKeyRange(21, 108);
	else KeyboardPainterManager.setKeyRange(sequence.getMinNote(), sequence.getMaxNote());
    }
    public static RGB getTrackColor(int trackNum) {
	if (trackNum < 0 || trackNum >= trackColors.length) return defaultColor;
	else return trackColors[trackNum];
    }
    public static void setTrackColor(RGB color, int trackNum) {
	try {
	    if (trackNum == -1) {
		defaultColor = color;
		Config.xmlConfig.setProperty("InputColor.red", color.red);
		Config.xmlConfig.setProperty("InputColor.green", color.green);
		Config.xmlConfig.setProperty("InputColor.blue", color.blue);
		return;
	    }
	    trackColors[trackNum] = color;
	} catch (Exception e) {
	    System.out.println("PianoAbstract.setTrackColor()" + ": trackNum = " + trackNum + " out of range");
	}
    }
}
