/*
 *	Programmer:	Piotr Borek
 *	E-mail:		piotrborek@op.pl
 *	Copyright 2010  Piotr Borek
 *
 *	Distributed under the terms of the GPL (GNU Public License)
 *
 *   Jarre is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.app.widgets;

import jarre.widgets.PlayerButtons;
import jarre.widgets.PlayerProgressBar;
import jarre.widgets.PlayerButtons.ActionListener;
import jarre.widgets.PlayerButtons.IWidget;
import jarre.widgets.utils.GridLayoutFactory;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Listener;

public class PlayerWidget extends Composite {
    public PlayerWidget(Composite parent, int style) {
	this(parent, style, null);
    }
    public PlayerWidget(Composite parent, int style, String[] shortcuts) {
	super(parent, style);
	GridLayoutFactory.swtDefaults().applyTo(this);
	progressBar = new PlayerProgressBar(this, SWT.NONE);
	progressBar.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
	playerButtons = new PlayerButtons(this, SWT.NONE, shortcuts);
    }
    @Override public void setEnabled(boolean enabled) {
	progressBar.setEnabled(enabled);
	playerButtons.setEnabled(enabled);
    }
    public IWidget[] createButtons(Composite parent, Class<?> clazz) {
	return playerButtons.createButtons(parent, clazz);
    }
    public void setPlay(boolean p) {
	playerButtons.setPlay(p);
    }
    public void setSongTime(int seconds) {
	progressBar.setSongTime(seconds);
    }
    public void setCurrentTime(int seconds) {
	progressBar.setCurrentTime(seconds);
    }
    public void addProgressBarListener(Listener listener) {
	progressBar.addSelectionListener(listener);
    }
    public void addActionListener(ActionListener listener) {
	playerButtons.addActionListener(listener);
    }
    private PlayerProgressBar progressBar;
    private PlayerButtons playerButtons;
}
