/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.app.widgets;

import static jarre.i18n.I18n._;
import jarre.Jarre;
import jarre.midi.MidiSystem;
import jarre.midi.MySequence;
import jarre.midi.event.EventEnum;
import jarre.midi.event.EventMessage;
import jarre.midi.event.NoteOffEvent;
import jarre.midi.event.NoteOnEvent;
import jarre.widgets.ColorMenu;
import jarre.widgets.ColorWidget;
import jarre.widgets.common.views.IControlProvider;
import jarre.widgets.common.views.IEnabledProvider;
import jarre.widgets.common.views.ILabelProvider;
import jarre.widgets.common.views.ITableSelectionListener;
import jarre.widgets.common.views.ItemEnum;
import jarre.widgets.utils.EventSlot;
import jarre.widgets.views.table.TableColumn2;
import jarre.widgets.views.table.TableView2;

import java.util.ArrayList;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

public class TracksWidget extends Composite {
    private Model[] model;
    private ColorMenu colorMenu;
    private TableView2 tableView;
    private MySequence sequence;
    private EventSlot<Listener, Event> changeSlot = new EventSlot<Listener, Event>() {
	@Override public void handleEvent(Listener listener, Event arg) {
	    listener.handleEvent(arg);
	}
    };
    public TracksWidget(Composite parent, int style) {
	super(parent, style);
	setLayout(new FillLayout());
	colorMenu = new ColorMenu(getShell());
	tableView = new TableView2(this, SWT.V_SCROLL);
	tableView.setLabelProvider(new ILabelProvider() {
	    @Override public boolean getBoolValue(int row, int column) {
		Model m = model[row];
		if (m.trackNum == -1) return false;
		switch (column) {
		    case 0:
			return m.visible;
		    case 1:
			return m.playInBackground;
		    default:
			return false;
		}
	    }
	    @Override public CharSequence getText(int row, int column) {
		Model m = model[row];
		switch (column) {
		    case 2:
			if (m.trackNum == -1) return _("Input");
			else return _("Track") + " " + row;
		    default:
			return null;
		}
	    }
	    @Override public String getToolTip(int row, int column) {
		switch (column) {
		    case 0:
			return _("Show track.");
		    case 1:
			return _("Play track in background.");
		    case 2:
			return _("Track name.");
		    case 3:
			return _("Track color.");
		    default:
			return null;
		}
	    }	    
	});
	tableView.setControlProvider(new IControlProvider() {
	    @Override public Control createInstance(Composite parent, int row, int column) {
		Model m = model[row];
		if (m.trackNum == -1) return createInputItem(parent, PianoAbstract.getTrackColor(-1));
		else return createColorWidget(parent, m.color, m.trackNum);
	    }
	});
	tableView.setEnabledProvider(new IEnabledProvider() {
	    @Override public boolean getEnabled(int row, int column) {
		Model m = model[row];
		if (m.trackNum == -1 && (column == 0 || column == 1)) return false;
		return true;
	    }
	});
	tableView.addSelectionListener(new ITableSelectionListener() {
	    @Override public void handleEvent(TableView2 view, Control[] items, int index, int column) {
		if (index < 0) return;
		Button button;
		Model m = model[index];
		switch (column) {
		    case 0:
			button = (Button)items[column];
			m.visible = button.getSelection();
			if (m.visible) m.playInBackground = false;
			doUpdate(m);
			break;
		    case 1:
			button = (Button)items[column];
			m.playInBackground = button.getSelection();
			if (m.playInBackground) m.visible = false;
			doUpdate(m);
			break;
		}
	    }
	    private void doUpdate(Model m) {
		if (! m.visible && ! m.playInBackground) {
		    for (EventMessage message: sequence.getSequenceHelper().getActiveMessages()) {
			if (message.type == EventEnum.NOTE_ON) {
			    NoteOnEvent noteOn = message.getEvent();
			    if (noteOn.getTrackNum() == m.trackNum) {
				NoteOffEvent noteOff = new NoteOffEvent(noteOn);
				MidiSystem.channelSoundOff(noteOff.getChannel());
				Jarre.window.getPianoKeyboard().setNoteOff(noteOff);
				Jarre.window.getPianoKeyboard().redrawAndUpdate();
			    }
			}
		    }
		}
		sequence.setPlayBackground(m.trackNum, m.playInBackground);
		sequence.setTrackVisible(m.trackNum, m.visible);
		sendChangeEvent(null);
		tableView.updateData();
	    }
	});
	new TableColumn2(tableView, ItemEnum.CHECK_BUTTON);
	new TableColumn2(tableView, ItemEnum.CHECK_BUTTON);
	new TableColumn2(tableView, ItemEnum.LABEL);
	new TableColumn2(tableView, ItemEnum.CONTROL).setAllignment(SWT.FILL);
	/*
	TableColumn2 column = new TableColumn2(tableView, ItemEnum.CHECK_BUTTON);
	column.setText("1");
	column = new TableColumn2(tableView, ItemEnum.CHECK_BUTTON);
	column.setText("2");
	column = new TableColumn2(tableView, ItemEnum.LABEL);
	column.setText("3");
	column = new TableColumn2(tableView, ItemEnum.CONTROL);
	column.setText("4");
	column.setAllignment(SWT.FILL);
	tableView.setHeaderVisible(true);
	*/
	model = new Model[] { new Model(-1, null) };
	tableView.setItemsCount(1);
    }
    public void setSequence(MySequence sequence) {
	this.sequence = sequence;
	int numTracks = sequence.getTracks().length;
	ArrayList<Model> list = new ArrayList<Model>(numTracks);
	int row = 0;
	for (int i = 0; i < numTracks; i++)
	    if (sequence.hasNotes(i)) {
		RGB color = Jarre.colors[row++ % Jarre.colors.length];
		PianoAbstract.setTrackColor(color, i);
		list.add(new Model(i, color));
	    }
	list.add(new Model(-1, null));
	model = list.toArray(new Model[0]);
	tableView.setItemsCount(model.length);
    }
    public void addTrackChangeListener(Listener listener) {
	changeSlot.addListener(listener);
    }
    private void sendChangeEvent(Event event) {
	changeSlot.notifyListeners(event);
    }
    private Control createInputItem(Composite parent, RGB color) {
	final ColorWidget colorWidget = new ColorWidget(parent, SWT.NONE);
	colorWidget.setColor(new Color(parent.getDisplay(), color));
	colorWidget.addListener(SWT.MouseDown, new Listener() {
	    @Override public void handleEvent(Event event) {
		ColorDialog dialog = new ColorDialog(getShell(), SWT.NONE);
		dialog.setRGB(PianoAbstract.getTrackColor(-1));
		RGB color = dialog.open();
		if (color != null) {
		    colorWidget.setColor(color.red, color.green, color.blue);
		    colorWidget.redraw();
		    colorWidget.update();
		    PianoAbstract.setTrackColor(color, -1);
		}
	    } 
	});
	return colorWidget;
    }
    private ColorWidget createColorWidget(Composite parent, RGB color, final int trackNum) {
	final ColorWidget colorWidget = new ColorWidget(parent, SWT.NONE);
	colorWidget.setColor(new Color(parent.getDisplay(), color));
	colorWidget.addListener(SWT.MouseDown, new Listener() {
	    @Override public void handleEvent(Event event) {
		Listener listener = new Listener() {
		    @Override public void handleEvent(Event event) {
			RGB color = colorMenu.getSelectedColor().getRGB();
			colorWidget.setColor(color);
			PianoAbstract.setTrackColor(color, trackNum);
			colorWidget.redraw();
			colorMenu.removeListener(this);
			changeSlot.notifyListeners(event);
		    }
		};
		colorMenu.addListener(listener);
		Point pos = getDisplay().map(colorWidget, null, event.x, event.y);
		colorMenu.setLocation(pos.x, pos.y);
		colorMenu.open();
	    }
	});
	return colorWidget;
    }
    
    private class Model {
	public boolean visible = true;
	public boolean playInBackground = false;
	public RGB color;
	public int trackNum;
	public Model(int trackNum, RGB color) {
	    this.trackNum = trackNum;
	    this.color = color;
	}
    }
}
