/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.midi;

import jarre.midi.alsa.AlsaMidiDevice;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.sound.midi.MidiDevice;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Sequencer;
import javax.sound.midi.Synthesizer;

import org.eclipse.swt.SWT;

public class MidiDriver {
    public final static String MIDI_JAVA = "java";
    public final static String MIDI_ALSA = "alsa";
    private static HashMap<String, ArrayList<MidiDevice>> inputDevices = new HashMap<String, ArrayList<MidiDevice>>();
    private static HashMap<String, ArrayList<MidiDevice>> outputDevices = new HashMap<String, ArrayList<MidiDevice>>();
    public static void initialize() {
	initializeJava();
	if (SWT.getPlatform().equals("gtk"))
	    initializeAlsa();
    }
    public static List<MidiDevice> getInDevices(String driver) {
	return inputDevices.get(driver);
    }
    public static List<MidiDevice> getOutDevices(String driver) {
	return outputDevices.get(driver);
    }
    public static MidiDevice getInputByName(String driver, String name) {
	return getDeviceByName(inputDevices.get(driver), name);
    }
    public static MidiDevice getOutputByName(String driver, String name) {
	return getDeviceByName(outputDevices.get(driver), name);
    }
    public static void addDriver(String driver, ArrayList<MidiDevice> inputArray, ArrayList<MidiDevice> outputArray) {
	inputDevices.put(driver, inputArray);
	outputDevices.put(driver, outputArray);
    }
    private static MidiDevice getDeviceByName(List<MidiDevice> list, String name) {
	if (list == null) return null;
	for (MidiDevice dev: list)
	    if (dev.getDeviceInfo().getName().equals(name)) return dev;
	return null;	
    }
    private static void initializeJava() {
	ArrayList<MidiDevice> inputArray = new ArrayList<MidiDevice>();
	ArrayList<MidiDevice> outputArray = new ArrayList<MidiDevice>();
	addDriver(MIDI_JAVA, inputArray, outputArray);
	MidiDevice.Info[] info = MidiSystem.getMidiDeviceInfo();
	for (MidiDevice.Info i: info) {
	    MidiDevice device = null;
	    try {
		device = MidiSystem.getMidiDevice(i);
	    } catch (MidiUnavailableException e) { }
	    if (!(device instanceof Synthesizer) && !(device instanceof Sequencer))
	    {
		if (device.getMaxTransmitters() != 0) inputArray.add(device);
		if (device.getMaxReceivers() != 0) outputArray.add(device);
	    }
	}
    }
    private static void initializeAlsa() {
	AlsaMidiDevice.initialize();
	ArrayList<MidiDevice> inputArray = new ArrayList<MidiDevice>();
	ArrayList<MidiDevice> outputArray = new ArrayList<MidiDevice>();
	addDriver(MIDI_ALSA, inputArray, outputArray);
	for (MidiDevice device: AlsaMidiDevice.getDevices()) {
	    if (device.getMaxTransmitters() != 0) inputArray.add(device);
	    if (device.getMaxReceivers() != 0) outputArray.add(device);	    
	}
    }
}
