/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.midi;

import jarre.Jarre;
import jarre.app.widgets.PianoKeyboard;
import jarre.midi.sequencer.PianoSequencer;

import javax.sound.midi.MidiMessage;
import javax.sound.midi.Receiver;
import javax.sound.midi.ShortMessage;

public class PianoReceiver implements Receiver {
    private PianoKeyboard keyboard;
    private PianoSequencer pianoSequencer;
    public PianoReceiver(PianoKeyboard keyboard, PianoSequencer pianoSequencer) {
	this.keyboard = keyboard;
	this.pianoSequencer = pianoSequencer;
    }
    @Override public void close() {
    }
    @Override public void send(MidiMessage message, long timeStamp) {
	if (message instanceof ShortMessage) {
	    ShortMessage msg = (ShortMessage)message;
	    switch (msg.getCommand()) {
		case ShortMessage.NOTE_ON:
		    if (msg.getData2() != 0) {
			keyboard.setRawNoteOn(msg.getData1());
			redrawKeyboard();
			break;
		    }
		case ShortMessage.NOTE_OFF:
		    keyboard.setRawNoteOff(msg.getData1());
		    redrawKeyboard();
		    break;
	    }
	}
    }
    private void redrawKeyboard() {
	if (Jarre.window.getShell().isDisposed()) return;
	Jarre.window.syncExec(new Runnable() {
	    @Override public void run() {
		keyboard.redraw();
		keyboard.update();
		if (pianoSequencer.getFollowMode()) pianoSequencer.noteChangeSignal();
	    }
	});
    }
}
