/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.midi.event;

import jarre.midi.MySequence;

import javax.sound.midi.InvalidMidiDataException;
import javax.sound.midi.ShortMessage;

public class NoteOffEvent extends NoteMessageEvent {
    public NoteOffEvent(NoteOnEvent noteOnEvent) {
	super(noteOnEvent.getSequence(), createNoteOffMessage(noteOnEvent), noteOnEvent.getTrackNum(), noteOnEvent.getTick());
	this.noteOnEvent = noteOnEvent;
    }
    public NoteOffEvent(MySequence sequence, ShortMessage shortMessage, NoteOnEvent noteOnEvent, int trackNum, int tick) {
	super(sequence, shortMessage, trackNum, tick);
	this.noteOnEvent = noteOnEvent;
    }
    public NoteOnEvent getNoteOnEvent() {
        return noteOnEvent;
    }
    @Override public String toString() {
	return "NoteOffEvent " + getNote();
    }
    private static ShortMessage createNoteOffMessage(NoteOnEvent noteOnEvent) {
	ShortMessage message = new ShortMessage();
	try {
	    ShortMessage noteOnMessage = noteOnEvent.getMidiMessage();
	    message.setMessage(ShortMessage.NOTE_OFF,
	    	noteOnMessage.getChannel(),
	    	noteOnMessage.getData1(),
	    	noteOnMessage.getData2());
	} catch (InvalidMidiDataException e) {
	    System.out.println("Error. Can't be here\nNoteOffEvent line 53");
	    e.printStackTrace();
	}
	return message;
    }
    private NoteOnEvent noteOnEvent;
}
