/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.midi.event;

import jarre.midi.MySequence;

import javax.sound.midi.InvalidMidiDataException;
import javax.sound.midi.ShortMessage;

public class NoteOnEvent extends NoteMessageEvent {
    public NoteOnEvent(MySequence sequence, ShortMessage shortMessage, int trackNum, int tick, int length) {
	super(sequence, shortMessage, trackNum, tick);
	velocity = shortMessage.getData2();
	this.length = length;
    }
    @Override public void setMidiMessage(ShortMessage midiMessage) {
	super.setMidiMessage(midiMessage);
	velocity = midiMessage.getData2();
    }
    public int getVelocity() {
        return velocity;
    }
    public void setVelocity(int velocity) {
        this.velocity = velocity;
        setMidiMessage(changeVelocity(getMidiMessage(), velocity));
    }
    public int getLength() {
        return length;
    }
    public void setLength(int length) {
        this.length = length;
    }
    @Override public String toString() {
	return "NoteOnEvent " + getNote() + ", " + velocity + ", " + length;
    }
    protected ShortMessage changeVelocity(ShortMessage midiMessage, int velocity) {
	ShortMessage newMessage = new ShortMessage();
	try {
	    newMessage.setMessage(midiMessage.getCommand(),
	    	                  midiMessage.getChannel(),
	    	                  midiMessage.getData1(),
	    	                  velocity
	    	);
	} catch (InvalidMidiDataException e) {
	    throw new RuntimeException(e);
	}
	return newMessage;
    }
    private int velocity;
    private int length;
}
