/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.midi.event;

import jarre.midi.MySequence;

import javax.sound.midi.MetaMessage;

public class TempoChangeEvent extends NoteEvent {
    public TempoChangeEvent(MySequence sequence, MetaMessage metaMessage, MetaMessage previousMetaMessage, int trackNum, int tick) {
	super(sequence, trackNum, tick);
	tempo = calcTempo(metaMessage);
	if (previousMetaMessage == null) previousTempo = 500000;
	else previousTempo = calcTempo(previousMetaMessage);
    }
    public int getTempo() {
        return tempo;
    }
    public int getPreviousTempo() {
        return previousTempo;
    }
    public static int calcTempo(MetaMessage metaMessage) {
	byte[] b = metaMessage.getData();
	return ((b[0] & 0xFF) << 16) + ((b[1] & 0xFF) << 8) + (b[2] & 0xFF);
    }
    @Override public String toString() {
	return "TempoChangeEvent " + tempo;
    }
    private int tempo;
    private int previousTempo;
}
