/*
 *	Programmer:	Piotr Borek
 *	E-mail:		piotrborek@op.pl
 *	Copyright 2009  Piotr Borek
 *
 *	Distributed under the terms of the GPL (GNU Public License)
 *
 *   Gravity Wars is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.reflect;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.util.HashMap;

class Proxy implements InvocationHandler {
    private HashMap<String, Object> client;
    private Class<?> i;
    private Object object;
    private Proxy(HashMap<String, Object> client, Class<?> i) {
	this.client = client;
	this.i = i;
    }
    private Proxy(HashMap<String, Object> client, Class<?> i, Object object) {
	this(client, i);
	this.object = object;
    }
    @SuppressWarnings("unchecked")
    public static <T> T newInstance(HashMap<String, Object> client, Class<?> i) {
	if (! i.isInterface()) throw new RuntimeException("Need interface.");
	return (T)java.lang.reflect.Proxy.newProxyInstance(
	    i.getClassLoader(), 
	    new Class[] { i },
	    new Proxy(client, i));
    }
    public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
	if (method.getName().equals("getInstance")) return object;
	Object o = client.get(method.getName());
	if (o instanceof Constructor<?>) {
	    Constructor<?> c = (Constructor<?>)o;
	    object = c.newInstance(args);
	    return newInstance(client, i, object);
	} else if (o instanceof Method) {
	    Method m = (Method)o;
	    return m.invoke(object, args);
	} else if (method.getName().equals("toString")) {
	    return method.invoke(object, args);
	}
	throw new RuntimeException("Should never be here.");
    }
    @SuppressWarnings("unchecked")
    protected static <T> T newInstance(HashMap<String, Object> client, Class<?> i, Object object) {
	if (! i.isInterface()) throw new RuntimeException("Need interface.");
	return (T)java.lang.reflect.Proxy.newProxyInstance(
	    i.getClassLoader(), 
	    new Class[] { i },
	    new Proxy(client, i, object));
    }
}
