/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.test;

import jarre.midi.MySequence;
import jarre.midi.MyTrack;
import jarre.midi.event.EventEnum;
import jarre.midi.event.EventMessage;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;

import javax.sound.midi.InvalidMidiDataException;
import javax.sound.midi.MetaMessage;
import javax.sound.midi.MidiDevice;
import javax.sound.midi.MidiEvent;
import javax.sound.midi.MidiMessage;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Sequence;
import javax.sound.midi.Sequencer;
import javax.sound.midi.ShortMessage;
import javax.sound.midi.Track;

public class Test {
    public static void test4() {
	System.out.println("GO");
	try {
	    PrintWriter out = new PrintWriter("test.txt");
	    long x = System.currentTimeMillis();
	    MySequence sequence = MySequence.create(new FileInputStream("samplemidi/Requiem_for_a_Dream_(difficult).mid"));
	    System.out.println(System.currentTimeMillis() - x);
	    for (MyTrack track: sequence.getMyTracks()) {
		out.println("BEGIN TRACK");
		int index = 0;
		for (int k = 0; k < track.length; k++) {
		    EventMessage[] messages = track.getMessages(k);
		    for (EventMessage m: messages) {
			//if (m.type == EventEnum.NOTE_ON || m.type == EventEnum.NOTE_OFF || m.type == EventEnum.NOTE_ON_CONTINUE) {
			if (m.type != EventEnum.NONE) {
			    out.print(index + ": ");
			    if (m != null) out.print(m);
			    out.println("");
			}
		    }
		    index++;
		}
	    }
	    out.close();
	} catch (Exception e) {
	    e.printStackTrace();
	}
	System.out.println("END");
    }
    public static void test3() {
	MidiDevice.Info[] info = MidiSystem.getMidiDeviceInfo();
	int j = 0;
	for (MidiDevice.Info i: info)
	    System.out.println("" + j++ + " " + i.getName() + " " + i.getDescription() + " " + i.getVendor());
	Sequencer sequencer = null;
	//Synthesizer synth;
	MidiDevice device1 = null;
	MidiDevice device2 = null;
	try {
	    device1 = MidiSystem.getMidiDevice(info[37]);
	    device2 = MidiSystem.getMidiDevice(info[69]); //37
	    System.out.println(device1.getMaxReceivers());
	    System.out.println(device1.getMaxTransmitters());
	    System.out.println(device2.getMaxReceivers());
	    System.out.println(device2.getMaxTransmitters());
	    //synth = MidiSystem.getSynthesizer();
	    sequencer = MidiSystem.getSequencer(false);
	    System.out.println(sequencer.getTransmitter());
	    device1.open();
	    device2.open();
	    sequencer.open();
	    //synth.open();
	    //sequencer.getTransmitter().setReceiver(device1.getReceiver());
	    sequencer.getTransmitter().setReceiver(device2.getReceiver());
	    FileInputStream is = new FileInputStream("samplemidi/Requiem_for_a_Dream_(difficult).mid");
	    Sequence mySeq = MidiSystem.getSequence(is);
	    sequencer.setSequence(mySeq);
	    sequencer.start();
	} catch (MidiUnavailableException e) {
	    e.printStackTrace();
	} catch (FileNotFoundException e) {
	    e.printStackTrace();
	} catch (InvalidMidiDataException e) {
	    e.printStackTrace();
	} catch (IOException e) {
	    e.printStackTrace();
	} finally {
	    //sequencer.close();
	    //device2.close();
	}
    }
    public static void test2() {
	try {
	    Sequencer sequencer = MidiSystem.getSequencer();
	    System.out.println(sequencer.getDeviceInfo().getName());
	    System.out.println(sequencer.getDeviceInfo().getDescription());
	    if (sequencer == null)
		throw new MidiUnavailableException();
	    sequencer.open();
	    FileInputStream is = new FileInputStream("samplemidi/Requiem_for_a_Dream_(difficult).mid");
	    Sequence mySeq = MidiSystem.getSequence(is);
	    sequencer.setSequence(mySeq);
	    sequencer.start();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
    public static void test1() {
	String fileName = "samplemidi/KisstheRain.mid";
	File midiFile = new File( fileName );
	System.out.println( "Sequence file name: " + fileName );
	// Using MidiSystem, convert the file to a sequence.
	Sequence sequence = null;
	try {
	    sequence = MidiSystem.getSequence( midiFile );
	} catch (InvalidMidiDataException e) {
	    e.printStackTrace();
	} catch (IOException e) {
	    e.printStackTrace();
	}
	if (sequence != null) {
	   // Print sequence information
	   System.out.println( " length: " + 
	     sequence.getTickLength() + " ticks" );
	   System.out.println( " duration: " + 
	     sequence.getMicrosecondLength() + 
	     " micro seconds" );
	   System.out.println( " division type: " +  sequence.getDivisionType());
	   System.out.println( " resolution   : " + sequence.getResolution());
	}
	// Print track information
	Track [] tracks = sequence.getTracks();
	if ( tracks != null ) {
	   for ( int i = 0; i < tracks.length; i++ ) {
	      System.out.println( "Track " + i + ":" );
	      Track track = tracks[ i ];
	      for ( int j = 0; j < track.size(); j++ ) {
		  MidiEvent event = track.get( j );
		  MidiMessage message = event.getMessage();
		  System.out.println(" tick "+ event.getTick() + ", " );
		  if (message instanceof MetaMessage) {
		      MetaMessage meta = (MetaMessage)message;
		      System.out.println("META = " + meta.getType());
		  } else if (message instanceof ShortMessage) {
		      ShortMessage sh = (ShortMessage)message;
		      System.out.println("channel = " + sh.getChannel() + ", command = " + sh.getCommand());
		  }
	      } // for
	   } // for
	} // if
    }
}
