/*
 *	Programmer:	Piotr Borek
 *	E-mail:		piotrborek@op.pl
 *	Copyright 2010  Piotr Borek
 *
 *	Distributed under the terms of the GPL (GNU Public License)
 *
 *   Jarre is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.widgets;

import static jarre.i18n.I18n._;
import jarre.widgets.utils.Animation;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.PaletteData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Layout;

public class Composite2 {
    private static Animation animation = new Animation(48, 48, "data/process-working.png", SWT.COLOR_WIDGET_BACKGROUND);
    public Composite2(Composite parent, int style) {
	rootComposite = new Composite(parent, SWT.NONE); 
	rootComposite.setLayout(stackLayout = new StackLayout());
	controlComposite = new Composite(rootComposite, SWT.NONE);
	showOnTop(controlComposite);
    }
    public void setLayout(Layout layout) {
	controlComposite.setLayout(layout);
    }
    public void setLayoutData(Object layoutData) {
	rootComposite.setLayoutData(layoutData);
    }
    public Composite getRoot() {
	return rootComposite;
    }
    public Composite getComposite() {
	return controlComposite;
    }
    public void doWait(boolean wait) {
	if (wait) {
	    showOnTop(createWaitControl(rootComposite));
	} else {
	    Control control = stackLayout.topControl;
	    showOnTop(controlComposite);
	    control.dispose();
	}
    }
    public void doTask(final Runnable runnable) {
	doWait(true);
	new Thread() {
	    @Override public void run() {
		runnable.run();
		rootComposite.getDisplay().syncExec(new Runnable() {
		    @Override public void run() {
			doWait(false);
		    }		    
		});
	    }
	}.start();
    }
    protected Composite createWaitControl(final Composite parent) {
	Composite composite = new Composite(parent, SWT.NONE);
	composite.setLayout(new GridLayout());
	Image image = new Image(parent.getDisplay(), parent.getSize().x, parent.getSize().y);
	GC gc = new GC(parent);
	gc.copyArea(image, 0, 0);
	ImageData imageData = image.getImageData();
	PaletteData palette = imageData.palette;
	for (int x = 0; x < imageData.width; x++)
	    for (int y = 0; y < imageData.height; y++) {
		int pixel = imageData.getPixel(x, y);
		int r = ((pixel & palette.redMask) >>> 1) & palette.redMask;
		int g = ((pixel & palette.greenMask) >>> 1) & palette.greenMask;
	    	int b = ((pixel & palette.blueMask) >>> 1) & palette.blueMask;
	    	imageData.setPixel(x, y, r + g + b);
	    }
	final Image background = new Image(parent.getDisplay(), imageData);
	gc.dispose();
	image.dispose();
	composite.setBackgroundImage(background);
	Composite box = new Composite(composite, SWT.NONE);
	box.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, true));
	box.setLayout(new GridLayout());
	Label label = new Label(box, SWT.CENTER);
	label.setText(_("Please wait. Loading..."));
	final Label aniWidget = new Label(box, SWT.NONE);
	aniWidget.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, true));
	aniWidget.setImage(animation.getImage(0));
	parent.getDisplay().timerExec(30, new Runnable() {
	    private final int length = animation.getSize();
	    private int pos = 1;
	    @Override public void run() {
		if (! aniWidget.isDisposed()) {
		    if ((pos % length) == 0) pos = 1;
		    aniWidget.setImage(animation.getImage(pos++));
		    aniWidget.redraw();
		    aniWidget.update();
		    parent.getDisplay().timerExec(30, this);
		}
	    }
	});
	return composite;
    }
    protected void showOnTop(Composite composite) {
	stackLayout.topControl = composite;
	controlComposite.layout();
	rootComposite.layout(true, true);
    }
    private Composite rootComposite;
    private Composite controlComposite;
    private StackLayout stackLayout;
}
