/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.widgets;

import jarre.widgets.utils.GridLayoutFactory;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

public class WidgetMenu {
    private Shell shell;
    private Control selected;
    private List<Listener> listeners = new ArrayList<Listener>();
    public WidgetMenu(Shell parent) {
	shell = new Shell(parent, SWT.TOOL);
	shell.addListener(SWT.MouseDown, new CloseShell());
	shell.addListener(SWT.Deactivate, new CloseShell());
	shell.setBackground(shell.getDisplay().getSystemColor(SWT.COLOR_WIDGET_FOREGROUND));
	GridLayoutFactory.fillDefaults().setSpacing(2).applyTo(shell);
    }
    public void open() {
	shell.open();
    }
    public void addListener(Listener listener) {
	listeners.add(listener);
    }
    public void removeListener(Listener listener) {
	listeners.remove(listener);
    }
    public void addWidgetItem(final Control control) {
	GridData gData = new GridData(SWT.FILL, SWT.FILL, true, true);
	gData.minimumWidth = 96;
	gData.minimumHeight = 20;
	control.setBackground(shell.getBackground());
	control.setLayoutData(gData);
	control.addListener(SWT.MouseDown, new CloseShell());
	control.addListener(SWT.MouseDown, new Listener() {
	    @Override public void handleEvent(Event event) {
		selected = control; 
		// handle event on list copy
		for (Listener listener: listeners.toArray(new Listener[0]))
		    listener.handleEvent(event);
	    } 
	});
	control.addListener(SWT.MouseEnter, new Listener() {
	    @Override public void handleEvent(Event event) {
		control.setBackground(shell.getDisplay().getSystemColor(SWT.COLOR_LIST_SELECTION));
	    } 
	});
	control.addListener(SWT.MouseExit, new Listener() {
	    @Override public void handleEvent(Event event) {
		control.setBackground(shell.getBackground());
	    } 
	});
	shell.pack();
    }
    public void setLocation(int x, int y) {
	shell.setLocation(x, y);
    }
    public Control getSelected() {
	return selected;
    }
    public Shell getShell() {
	return shell;
    }
    private class CloseShell implements Listener {
	@Override public void handleEvent(Event event) {
	    shell.setVisible(false);
	}
    }
}
