/*
 *	Programmer:	Piotr Borek
 *	E-mail:		piotrborek@op.pl
 *	Copyright 2010  Piotr Borek
 *
 *	Distributed under the terms of the GPL (GNU Public License)
 *
 *   Jarre is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.widgets.internal;

import jarre.widgets.common.IImageFactory;

import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.PaletteData;
import org.eclipse.swt.widgets.Display;

public class GTKImageFactory implements IImageFactory {
    @Override public Image createImage (String name) {
	long pixbuf = GtkJNI.icon_set_render_icon (
	    GtkJNI.icon_factory_lookup_default(name),
	    GtkJNI.widget_get_default_style(),
	    GtkJNI.GTK_TEXT_DIR_NONE,
	    GtkJNI.GTK_STATE_NORMAL,
	    GtkJNI.GTK_ICON_SIZE_SMALL_TOOLBAR,
	    0,
	    null
	);
	if (pixbuf == 0) return null;
	int width = GtkJNI.pixbuf_get_width(pixbuf);
	int height = GtkJNI.pixbuf_get_height(pixbuf);
	int stride = GtkJNI.pixbuf_get_rowstride(pixbuf);
	boolean hasAlpha = GtkJNI.pixbuf_get_has_alpha(pixbuf);
	byte [] data = GtkJNI.pixbuf_get_pixels(pixbuf);
	GtkJNI.object_unref(pixbuf);
	ImageData imageData = null;
	if (hasAlpha) {
	    PaletteData palette = new PaletteData(0xFF000000, 0xFF0000, 0xFF00);
	    imageData = new ImageData(width, height, 32, palette);
	    byte[] alpha = new byte[stride * height];
	    for (int y = 0; y < height; y++) {
		for (int x = 0; x < width; x++) {
		    alpha[y*width+x] = data[y*stride+x*4+3];
		    data[y*stride+x*4+3] = 0;
		}
	    }
	    imageData.setAlphas(0, 0, width * height, alpha, 0);
	} else {
	    PaletteData palette = new PaletteData(0xFF0000, 0xFF00, 0xFF);
	    imageData = new ImageData(width, height, 24, palette);
	}
	imageData.data = data;
	imageData.bytesPerLine = stride;
	return new Image(Display.getCurrent(), imageData);
    }
}
