/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.widgets.utils;

import java.util.LinkedList;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Display;

public class Animation {
    private Image[] animation;
    public Animation(int width, int height, String filename, int background) {
	this(Display.getCurrent(), width, height, filename, Display.getCurrent().getSystemColor(background));
    }
    public Animation(Display display, int width, int height, String filename, Color background) {
	Image image = new Image(display, filename);
	Rectangle rect = image.getBounds();
	LinkedList<Image> imageList = new LinkedList<Image>();
	for (int y = 0; y < rect.height; y += height)
	    for (int x = 0; x < rect.width; x += width) {
		Image temp = new Image(display, width, height);
		GC gc = new GC(temp);
		gc.setBackground(background);
		gc.fillRectangle(0, 0, width, height);
		gc.drawImage(image, x, y, width, height, 0, 0, width, height);
		gc.dispose();
		imageList.add(temp);
	    }
	animation = imageList.toArray(new Image[0]);
    }
    public Image getImage(int i) {
	return animation[i];
    }
    public int getSize() {
	return animation.length;
    }
    public void dispose() {
	for(Image i: animation) i.dispose();
    }
}
