/*
 *	Programmer:	Piotr Borek
 *	E-mail:		piotrborek@op.pl
 *	Copyright 2010  Piotr Borek
 *
 *	Distributed under the terms of the GPL (GNU Public License)
 *
 *   Jarre is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.widgets.utils;

import java.util.LinkedHashSet;

public abstract class EventSlot<T, U> {
    public void addListener(T listener) {
	synchronized (listeners) {
	    listeners.add(listener);
	}
    }
    public void removeListener(T listener) {
	synchronized (listeners) {
	    if (listeners.contains(listener)) listeners.remove(listener);
	    else throw new RuntimeException("Cannot be here.");
	}
    }
    @SuppressWarnings("unchecked")
    public void notifyListeners(U arg) {
	Object[] array;
	synchronized (listeners) {
	    array = listeners.toArray(new Object[0]);
	}
	for (Object o: array)
	    handleEvent((T)o, arg);
    }
    protected abstract void handleEvent(T listener, U arg); 
    private LinkedHashSet<T> listeners = new LinkedHashSet<T>();
}
