/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.widgets.utils;

import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

public class GridLayoutFactory {
    GridLayout layout;
    public GridLayoutFactory(GridLayout layout) {
	this.layout = layout;
    }
    public void applyTo(Composite composite) {
	composite.setLayout(layout);
    }
    public GridLayout create() {
	return layout;
    }
    public GridLayoutFactory columns(int num) {
	layout.numColumns = num;
	return this;
    }
    public GridLayoutFactory setSpacing(int spacing) {
	layout.horizontalSpacing = spacing;
	layout.verticalSpacing = spacing;
	return this;
    }
    public GridLayoutFactory hSpace(int spacing) {
	layout.horizontalSpacing = spacing;
	return this;
    }
    public GridLayoutFactory vSpace(int spacing) {
	layout.verticalSpacing = spacing;
	return this;
    }
    public GridLayoutFactory marginHeight(int height) {
	layout.marginHeight = height;
	return this;
    }
    public GridLayoutFactory marginWidth(int width) {
	layout.marginWidth = width;
	return this;
    }
    public GridLayoutFactory equalWidth() {
	layout.makeColumnsEqualWidth = true;
	return this;
    }
    public static GridLayoutFactory swtDefaults() {
	return new GridLayoutFactory(new GridLayout());
    }
    public static GridLayoutFactory fillDefaults() {
	GridLayout layout = new GridLayout();
	layout.horizontalSpacing = 0;
	layout.verticalSpacing = 0;
	layout.marginHeight = 0;
	layout.marginWidth = 0;
	return new GridLayoutFactory(layout);
    }
}
