/*
 *	Programmer:	Piotr Borek
 *	E-mail:		piotrborek@op.pl
 *	Copyright 2010  Piotr Borek
 *
 *	Distributed under the terms of the GPL (GNU Public License)
 *
 *   Jarre is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.widgets.views;

import static jarre.i18n.I18n._;
import jarre.widgets.common.IImageFactory;
import jarre.widgets.common.views.ILabelProvider;
import jarre.widgets.common.views.ITableListener;
import jarre.widgets.common.views.ITableSelectionListener;
import jarre.widgets.common.views.ItemEnum;
import jarre.widgets.layout.CustomGridData;
import jarre.widgets.layout.CustomGridLayout;
import jarre.widgets.utils.ImageFactory;
import jarre.widgets.views.table.TableColumn2;
import jarre.widgets.views.table.TableSeparator2;
import jarre.widgets.views.table.TableView2;

import java.io.File;
import java.io.FileFilter;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

public class DirectoryView extends Composite {
    public DirectoryView(Composite parent, final int style, String startDir) {
	super(parent, style);
	setLayout(new CustomGridLayout(2, false));
	pathComposite = new Composite(this, SWT.NONE);
	pathComposite.setLayoutData(new CustomGridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
	pathComposite.setVisible(false);
	RowLayout layout = new RowLayout();
	layout.marginTop = 0;
	layout.marginBottom = 0;
	layout.marginLeft = 0;
	layout.marginRight = 0;
	layout.spacing = 0;
	pathComposite.setLayout(new RowLayout());
	locationComposite = new Composite(this, SWT.NONE);
	locationComposite.setLayoutData(new CustomGridData(SWT.FILL, SWT.FILL, false, true));
	locationComposite.setLayout(new GridLayout());
	locationComposite.setVisible(false);
	locationTable = new TableView2(locationComposite, SWT.BORDER | SWT.V_SCROLL);
	locationTable.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
	locationTable.setHeaderVisible(true);
	TableColumn2 column2 = new TableColumn2(locationTable, ItemEnum.LABEL);
	column2.setAllignment(SWT.FILL);
	column2.setText(_("Places"));
	locationTable.setLabelProvider(new ILabelProvider() {
	    @Override public boolean getBoolValue(int row, int column) {
		return false;
	    }
	    @Override public CharSequence getText(int row, int column) {
		Object o = locationList.get(row);
		if (o instanceof File) {
		    File file = (File) o;
		    String name = file.getName();
		    if (name.isEmpty()) return "/";
		    else return file.getName();
		} else if (o instanceof RootDrive) {
		    return ((RootDrive)o).file.getAbsolutePath();
		} else if (o instanceof String) {
		    return (String)o;
		} else if (o instanceof TableSeparator2) {
		    return TableView2.SEPARATOR;
		} else {
		    return null;
		}
	    }
	    @Override public String getToolTip(int row, int column) {
		return null;
	    }	    
	});
	locationList.add(_("Recently Used."));
	locationList.add(new TableSeparator2());
	if (SWT.getPlatform().equals("win32")) {
	    for (File file: File.listRoots())
		locationList.add(new RootDrive(file));
	    locationList.add(new TableSeparator2());
	    locationList.add(new File(System.getProperty("user.home")));
	    locationList.add(new TableSeparator2());
	} else {
	    locationList.add(new File("/"));
	    locationList.add(new File("/mnt"));
	    locationList.add(new File("/home"));
	    locationList.add(new File(System.getProperty("user.home")));
	    locationList.add(new TableSeparator2());
	}
	locationItemCount = locationList.size();
	refreshLocationTable();
	locationTable.addSelectionListener(new ITableSelectionListener() {
	    @Override public void handleEvent(TableView2 view, Control[] items, int index, int column) {
		removeButton.setEnabled(index >= locationItemCount);
	    }	    
	});
	locationTable.addDoubleClickListener(new ITableListener() {
	    @Override public void handleEvent(TableView2 view) {
		int index = view.getSelectionIndex();
		if (index < 0) return;
		if (index == 0) {
		    tmpFiles = files;
		    files = recentlyUsed.toArray(new File[0]);
		    directoryTable.setItemCount(files.length + 1);
		    directoryTable.clearAll();
		    path = null;
		    return;
		}
		Object object = locationList.get(index);
		if (object instanceof File) {
		    File file = (File) object;
		    setDirectory(file.getAbsolutePath());
		} else if (object instanceof RootDrive) {
		    RootDrive drive = (RootDrive) object;
		    setDirectory(drive.file.getAbsolutePath());
		}
	    }	    
	});
	Composite composite = new Composite(locationComposite, SWT.NONE);
	composite.setLayout(new FillLayout(SWT.HORIZONTAL));
	addButton = new Button(composite, SWT.PUSH);
	addButton.setText(_("Add"));
	addButton.setEnabled(false);
	addButton.addListener(SWT.Selection, new Listener() {
	    @Override public void handleEvent(Event event) {
		int index = directoryTable.getSelectionIndex();
		String path = files[index - 1].getAbsolutePath();
		userDirectories.add(path);
		locationList.add(new File(path));
		refreshLocationTable();
	    }
	});
	removeButton = new Button(composite, SWT.PUSH);
	removeButton.setText(_("Remove"));
	removeButton.setEnabled(false);
	removeButton.addListener(SWT.Selection, new Listener() {
	    @Override public void handleEvent(Event event) {
		int index = locationTable.getSelectionIndex();
		if (index < 0) return;
		File file = (File) locationList.get(index);
		locationList.remove(index);
		userDirectories.remove(file.getAbsolutePath());
		removeButton.setEnabled(false);
		refreshLocationTable();
	    } 
	});
	composite = new Composite(this, SWT.NONE);
	composite.setLayoutData(new CustomGridData(SWT.FILL, SWT.FILL, true, true));
	composite.setLayout(new CustomGridLayout());
	directoryTable = new Table(composite, SWT.SINGLE | SWT.FULL_SELECTION | SWT.BORDER | SWT.VIRTUAL);
	directoryTable.setLayoutData(new CustomGridData(SWT.FILL, SWT.FILL, true, true));
	directoryTable.addListener(SWT.SetData, new Listener() {
	    @Override public void handleEvent(Event event) {
		TableItem item = (TableItem)event.item;
		if (files == null) return;
		int index = directoryTable.indexOf(item);
		if (index == 0) {
		    item.setText(new String[] {"..", "", ""} );
		    item.setImage(upImage);
		    return;
		}
		File file = files[index - 1];
		item.setText(0, file.getName());
		item.setText(2, dateFormat.format(file.lastModified()));
		if (file.isDirectory()) {
		    item.setImage(dirImage);
		    item.setText(1, "");
		}
		else {
		    item.setImage(fileImage);
		    item.setText(1, convert(file.length()));
		}
	    }	    
	});
	directoryTable.setHeaderVisible(true);
	directoryTable.addListener(SWT.MouseDoubleClick, new Listener() {
	    @Override public void handleEvent(Event event) {
		if (files == null) return;
		int index = directoryTable.getSelectionIndex();
		if (index > 0) {
		    File file = files[index - 1];
		    setDirectory(file.getAbsolutePath());
		} else if (index == 0) {
		    if (path == null) {
			files = tmpFiles;
			directoryTable.setItemCount(files.length + 1);
			directoryTable.clearAll();
			return;
		    }
		    int i = path.lastIndexOf(File.separator);
		    String newPath = path.substring(0, i);
		    if (newPath.isEmpty()) newPath = File.separator;
		    setDirectory(newPath, false);
		}
	    }	    
	});
	directoryTable.addListener(SWT.Selection, new Listener() {
	    @Override public void handleEvent(Event event) {
		selection = null;
		if (files == null) return;
		int index = directoryTable.getSelectionIndex();
		if (index == 0) {
		    addButton.setEnabled(false);
		    return;
		}
		File file = files[index - 1];
		if (file.isDirectory()) {
		    addButton.setEnabled(index != 0);
		    if ((style & SWT.OPEN) == SWT.OPEN) extra.setEnabled(false);
		}
		else {
		    selection = file;
		    addButton.setEnabled(false);
		    if ((style & SWT.OPEN) == SWT.OPEN) extra.setEnabled(true);
		}
	    }
	});
	TableColumn column = new TableColumn(directoryTable, SWT.NONE);
	column.setText(_("Name"));
	column.setWidth(280);
	column.addListener(SWT.Selection, new ColumnSorterListener(new Comparator<File>() {
	    @Override public int compare(File a, File b) {
		if (a.isDirectory() && b.isDirectory())
		    return a.getName().compareTo(b.getName());
		if (a.isFile() && b.isFile())
		    return a.getName().compareTo(b.getName());
		if (a.isDirectory()) return -1;
		else return 1;
	    }
	}));
	column = new TableColumn(directoryTable, SWT.NONE);
	column.setText(_("Size"));
	column.setWidth(80);
	column.addListener(SWT.Selection, new ColumnSorterListener(new Comparator<File>() {
	    @Override public int compare(File a, File b) {
		if (a.isDirectory() && b.isDirectory())
		    return a.getName().compareTo(b.getName());
		if (a.isFile() && b.isFile()) {
		    int x = (int)(a.length() - b.length());
		    if (x == 0) return a.getName().compareTo(b.getName());
		    else return x;
		}
		if (a.isDirectory()) return -1;
		else return 1;
	    }
	}));
	column = new TableColumn(directoryTable, SWT.NONE);
	column.setText(_("Modified"));
	column.setWidth(80);
	column.addListener(SWT.Selection, new ColumnSorterListener(new Comparator<File>() {
	    @Override public int compare(File a, File b) {
		return (int)(a.lastModified() - b.lastModified());
	    }
	}));
	Composite c = new Composite(composite, SWT.NONE);
	c.setLayoutData(new CustomGridData(SWT.FILL, SWT.FILL, true, false));
	c.setLayout(new FillLayout());
	new Label(c, SWT.NONE); new Label(c, SWT.NONE); new Label(c, SWT.NONE);
	cancel = new Button(c, SWT.PUSH);
	cancel.setText(_("Cancel"));
	cancel.addListener(SWT.Selection, new Listener() {
	    @Override public void handleEvent(Event event) {
		selection = null;
	    } 
	});
	extra = new Button(c, SWT.PUSH);
	if ((style & SWT.OPEN) == SWT.OPEN) {
	    extra.setText(_("Open"));
	    extra.setEnabled(false);
	    extra.addListener(SWT.Selection, new Listener() {
		@Override public void handleEvent(Event event) {
		    if (selection == null) return;
		    for (File file: recentlyUsed)
			if (selection.getAbsolutePath().equals(file.getAbsolutePath()))
			    return;
		    if (recentlyUsed.size() > historySize) recentlyUsed.remove(0);
		    recentlyUsed.add(selection);
		}
	    });
	}
	if ((style & SWT.SAVE) == SWT.SAVE) extra.setText(_("Save"));
	setDirectory(startDir);
    }
    public void addSelectionListener(Listener listener) {
	directoryTable.addListener(SWT.Selection, listener);
    }
    public void showLocations(boolean showLocations) {
	locationComposite.setVisible(showLocations);
	getShell().layout(true, true);
    }
    public void showDirPath(boolean showDirPath) {
	pathComposite.setVisible(showDirPath);
	getShell().layout(true, true);
    }
    public void setUserDirectories(String[] args) {
	if (args == null) return;
	for (int i = locationList.size(); i > locationItemCount; i--) {
	    locationList.remove(i);
	}
	for (String name: args) {
	    userDirectories.add(name);
	    locationList.add(new File(name));
	}
	refreshLocationTable();
    }
    public String[] getUserDirectories() {
	return userDirectories.toArray(new String[0]);
    }
    public File getSelection() {
	return selection;
    }
    public void setFileFilter(String[] filter) {
	this.filter = filter;
	setDirectory(path, false);
    }
    public void setHistorySize(int historySize) {
	if (historySize < 1) return;
	this.historySize = historySize;
	while (recentlyUsed.size() > historySize) recentlyUsed.remove(0);
    }
    public void setRecentlyUsed(String[] args) {
	if (args == null) return;
	int i = args.length - historySize;
	if (i < 0) i = 0;
	for (; i < args.length; i++)
	    recentlyUsed.add(new File(args[i]));
    }
    public String[] getRecentlyUsed() {
	String[] files = new String[recentlyUsed.size()];
	for (int i = 0; i < files.length; i++)
	    files[i] = recentlyUsed.get(i).getAbsolutePath();
	return files;
    }
    public void setDirectory(String path, boolean dispose) {
	try {
	    this.path = path;
	    if (! dispose) return;
	    for (Control control: pathComposite.getChildren())
		control.dispose();
	    String[] array = null;
	    if (SWT.getPlatform().equals("win32")) {
		array =  path.split("\\\\");
	    } else {
		Button button = new Button(pathComposite, SWT.PUSH);
		button.setText("/");
		button.setData("jarre.widgets.views.path", "/");
		button.addListener(SWT.Selection, new ButtonListener());
		array = path.split("/");
	    }
	    String p = "";
	    for (String s: array) {
		if (s.isEmpty()) continue;
		Button button = new Button(pathComposite, SWT.PUSH);
		button.setText(s);
		p = p + File.separator + s;
		button.setData("jarre.widgets.views.path", p);
		button.addListener(SWT.Selection, new ButtonListener());
	    }
	} finally {
	    pathComposite.layout(true, true);
	    scanDirectory(path);
	    directoryTable.setSelection(0);
	    addButton.setEnabled(false);
	}	
    }
    public void setDirectory(String path) {
	setDirectory(path, true);
    }
    public String getDirectory() {
	return path;
    }
    public Button getCancelButton() {
	return cancel;
    }
    public Button getExtraButton() {
	return extra;
    }
    private void scanDirectory(String path) {
	File dir = new File(path);
	if (! dir.isDirectory()) return;
	if (filter == null) files = dir.listFiles();
	else {
	    files = dir.listFiles(new FileFilter() {
		@Override public boolean accept(File file) {
		    if (file.isDirectory()) return true;
		    for (String regex: filter)
			if (file.getName().matches(regex)) return true;
		    return false;
		}
	    });
	}   
	Arrays.sort(files, new Comparator<File>() {
	    @Override public int compare(File a, File b) {
		if (a.isDirectory() && b.isDirectory())
		    return a.getName().compareTo(b.getName());
		if (a.isFile() && b.isFile())
		    return a.getName().compareTo(b.getName());
		if (a.isDirectory()) return -1;
		else return 1;
	    }	    
	});
	directoryTable.setItemCount(files.length + 1);
	directoryTable.clearAll();
    }
    private String convert(long size) {
	if (size < 1024) return Long.toString(size);
	else if (size < 1024 * 1024) return Long.toString(size / 1024) + " kB";
	else if (size < 1024 * 1024 * 1024) return Long.toString(size / 1024 / 1024) + " MB";
	else return Long.toString(size / 1024 / 1024 / 1024) + " GB";
    }
    private void refreshLocationTable() {
	locationTable.setItemsCount(locationList.size());
	locationTable.updateData();
    }
    private File selection;
    private Button cancel;
    private Button extra;
    private Button addButton;
    private Button removeButton;
    private Table directoryTable;
    private TableView2 locationTable;
    private Composite locationComposite;
    private ArrayList<Object> locationList = new ArrayList<Object>();
    private int locationItemCount;
    private Composite pathComposite;
    private int historySize = 50;
    private String path;
    private String[] filter;
    private File[] files;
    private File[] tmpFiles;
    private ArrayList<File> recentlyUsed = new ArrayList<File>();
    private TableColumn lastSorted;
    private LinkedList<String> userDirectories = new LinkedList<String>();
    private static Image dirImage = ImageFactory.createImage(IImageFactory.IMAGE_DIRECTORY);
    private static Image fileImage = ImageFactory.createImage(IImageFactory.IMAGE_FILE);
    private static Image upImage = ImageFactory.createImage(IImageFactory.IMAGE_GO_UP);
    private static SimpleDateFormat dateFormat = new SimpleDateFormat("dd-MM-yyyy");
    private class ColumnSorterListener implements Listener {
	private boolean b = true;
	private Comparator<File> comparator;
	public ColumnSorterListener(Comparator<File> comparator) {
	    this.comparator = comparator;
	}
	@Override public void handleEvent(Event event) {
	    if (lastSorted == (TableColumn)event.widget) b = !b;
	    else b = true;
	    lastSorted = (TableColumn)event.widget;
	    if (b)
		Arrays.sort(files, comparator);
	    else 
		Arrays.sort(files, Collections.reverseOrder(comparator));
	    directoryTable.clearAll();	
	}
    }
    private class ButtonListener implements Listener {
	@Override public void handleEvent(Event event) {
	    Button button = (Button) event.widget;
	    setDirectory((String)button.getData("jarre.widgets.views.path"), false);
	}	
    }
    private class RootDrive {
	public final File file;
	public RootDrive(File file) {
	    this.file = file;
	}
    }
}
