/*
 *	Programmer:	Piotr Borek
 *	E-mail:		piotrborek@op.pl
 *	Copyright 2010  Piotr Borek
 *
 *	Distributed under the terms of the GPL (GNU Public License)
 *
 *   Jarre is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.widgets.views.table;

import jarre.widgets.common.views.ItemEnum;
import jarre.widgets.utils.Utils;

import java.util.Iterator;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

public class TableColumn2 implements Iterable<Control> {
    public TableColumn2(TableView2 parent, ItemEnum itemType) {
	this.parent = parent;
	this.itemType = itemType;
	colNumber = parent.columnList.size();
	parent.columnList.add(this);
	columnButton = new Button(parent.headerComposite, SWT.PUSH);
	Utils.disableFocus(columnButton);
    }
    public void setText(String text) {
	columnButton.setText(text);
    }
    public String getText() {
	return columnButton.getText();
    }
    public int getAllignment() {
        return flags;
    }
    public void setAllignment(int flags) {
        this.flags = flags;
    }
    public int getWidth() {
        return width;
    }
    public void setWidth(int width) {
	if (width < minWidth) this.width = minWidth;
	else this.width = width;
    }
    public int getMinWidth() {
        return minWidth;
    }
    public void setMinWidth(int minWidth) {
        this.minWidth = minWidth;
    }
    public int getLeftMargin() {
        return leftMargin;
    }
    public void setLeftMargin(int leftMargin) {
        this.leftMargin = leftMargin;
    }
    public int getRightMargin() {
        return rightMargin;
    }
    public void setRightMargin(int rightMargin) {
        this.rightMargin = rightMargin;
    }
    public int getColNumber() {
	return colNumber;
    }
    public ItemEnum getItemType() {
	return itemType;
    }
    public Button getColumnButton() {
	return columnButton;
    }
    @Override public Iterator<Control> iterator() {
	return new Iterator<Control>() {
	    private Iterator<TableRow2> it = parent.rowList.iterator();
	    @Override public boolean hasNext() {
		return it.hasNext();
	    }
	    @Override public Control next() {
		TableRow2 row = it.next();
		return row.getItems()[colNumber];
	    }
	    @Override public void remove() {
		throw new UnsupportedOperationException();
	    }	    
	};
    }
    Listener getMouseUpListener() {
	return mouseUpListener;
    }
    Listener getMouseDoubleClickListener() {
	return mouseDoubleClickListener;
    }
    private TableView2 parent;
    private int colNumber;
    private ItemEnum itemType;
    private int flags = SWT.LEFT;
    private int width;
    private int minWidth;
    private int leftMargin = 2;
    private int rightMargin = 2;
    private Button columnButton;
    private Listener mouseUpListener = new Listener() {
	@Override public void handleEvent(Event event) {
	    if (parent.selection == null) return;
	    parent.selectionSlot.notifyListeners(colNumber);
	}	
    };
    private Listener mouseDoubleClickListener = new Listener() {
	@Override public void handleEvent(Event event) {
	    parent.doubleClickSlot.notifyListeners(null);
	}	
    };
}
