/*
 *	Programmer:	Piotr Borek
 *	E-mail:		piotrborek@op.pl
 *	Copyright 2010  Piotr Borek
 *
 *	Distributed under the terms of the GPL (GNU Public License)
 *
 *   Jarre is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.widgets.views.table;

import jarre.widgets.common.views.IControlProvider;
import jarre.widgets.common.views.IEnabledProvider;
import jarre.widgets.common.views.ILabelProvider;
import jarre.widgets.common.views.ItemEnum;

import java.util.Arrays;
import java.util.Iterator;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

class TableRow2 implements Iterable<Control> {
    public TableRow2(TableView2 parent) {
	this.parent = parent;
    }
    public void initialize() {
	initialized = true;
	composite = new Composite(parent.main, SWT.NONE);
	separator = new Label(composite, SWT.SEPARATOR | SWT.HORIZONTAL);
	separator.setBackground(separator.getDisplay().getSystemColor(SWT.COLOR_LIST_BACKGROUND));
	separator.setVisible(false);
	TableColumn2[] columns = parent.getColumns();
	items = new Control[columns.length];
	for (int i = 0; i < items.length; i++) {
	    Control control = null;
	    TableColumn2 column = columns[i];
	    switch (column.getItemType()) {
	    	case CONTROL:
	    	    control = new Composite(composite, SWT.NONE);
	    	    ((Composite)control).setLayout(new FillLayout());
	    	    break;
	    	case LABEL:
	    	    control = new Label(composite, SWT.NONE);
	    	    break;
	    	case CHECK_BUTTON:
	    	    control = new Button(composite, SWT.CHECK);
	    	    break;
	    	default:
	    	    control = new Label(composite, SWT.NONE);
	    }
	    control.addListener(SWT.MouseUp, column.getMouseUpListener());
	    control.addListener(SWT.MouseDoubleClick, column.getMouseDoubleClickListener());
	    control.setBackground(parent.getDisplay().getSystemColor(SWT.COLOR_LIST_BACKGROUND));
	    items[i] = control;
	    composite.setBackground(parent.getDisplay().getSystemColor(SWT.COLOR_LIST_BACKGROUND));
	}	
    }
    public void setBackground(Color color) {
	TableColumn2[] columns = parent.getColumns();
	for (int i = 0; i < items.length; i++) {
	    Control control = items[i];
	    control.setBackground(color);
	    if (columns[i].getItemType() == ItemEnum.CONTROL)
		setBackground((Composite)control, color);
	}
	composite.setBackground(color);
    }
    public void fillWidthData(boolean recreateControls) {
	TableColumn2[] columns = parent.getColumns();
	IControlProvider provider1 = parent.getControlProvider();
	ILabelProvider   provider2 = parent.getLabelProvider();
	IEnabledProvider provider3 = parent.getEnabledProvider();
	setSeparator(false);
	if (provider2 != null) {
	    if (provider2.getText(index, 0) instanceof TableSeparator2) {
		setSeparator(true);
		return;
	    }
	}
	for (int i = 0; i < items.length; i++) {
	    Control control = items[i];
	    TableColumn2 column = columns[i];    
	    switch (column.getItemType()) {
	    	case CONTROL:
	    	    if (! recreateControls) break;
	    	    Composite area = (Composite) control;
	    	    for (Control c: area.getChildren()) c.dispose();
	    	    Control child = null;
	    	    if (provider1 != null) child = provider1.createInstance(area, index, i);
	    	    if (child == null) child = new Label(area, SWT.NONE);
	    	    area.layout(true, true);
	    	    if (selected) setBackground(area, area.getDisplay().getSystemColor(SWT.COLOR_LIST_SELECTION));
	    	    else setBackground(area, area.getDisplay().getSystemColor(SWT.COLOR_LIST_BACKGROUND));
	    	    if (provider2 != null) setToolTipText(area, provider2.getToolTip(index, i));
	    	    break;
	    	case LABEL:
	    	    Label label = (Label) control;
	    	    String text = null;
	    	    if (provider2 != null) text = getText(provider2, index, i);
	    	    if (text == null) text = "";
	    	    label.setText(text);
	    	    label.setAlignment(column.getAllignment());
	    	    break;
	    	case CHECK_BUTTON:
	    	    Button button = (Button) control;
	    	    if (provider2 != null) {
	    		button.setSelection(provider2.getBoolValue(index, i));
	    		text = getText(provider2, index, i);
	    		if (text == null) button.setText("");
	    		else button.setText(text);
	    	    }
	    	    else button.setSelection(false);
	    	    button.setAlignment(column.getAllignment());
	    	    break;
	    	default:
	    	    break;
	    }
	    if (provider3 != null)
		control.setEnabled(provider3.getEnabled(index, i));
	    if (provider2 != null)
		control.setToolTipText(provider2.getToolTip(index, i));
	}		
    }
    public void setVisible(boolean visible) {
	composite.setVisible(visible);
    }
    public void setSeparator(boolean visible) {
	isSeparator = visible;
	separator.setVisible(visible);
    }
    public boolean isSeparator() {
	return isSeparator;
    }
    public int getIndex() {
        return index;
    }
    public void setIndex(int index) {
        this.index = index;
    }
    public boolean isSelected() {
        return selected;
    }
    public void setSelected(boolean selected) {
	if (selected) setBackground(composite.getDisplay().getSystemColor(SWT.COLOR_LIST_SELECTION));
	else setBackground(composite.getDisplay().getSystemColor(SWT.COLOR_LIST_BACKGROUND));
        this.selected = selected;
    }
    public void setBounds(int x, int y, int width, int height) {
	if (isSeparator) separator.setBounds(0, 0, width, height);
	composite.setBounds(x, y, width, height);
    }
    public Rectangle getBounds() {
	return composite.getBounds();
    }
    public Point computeSize(int wHint, int hHint) {
	if (isSeparator) return separator.computeSize(wHint, hHint);
	else return composite.computeSize(wHint, hHint);
    }
    public Control[] getItems() {
	return items;
    }
    public int getHeight() {
        return height;
    }
    public void setHeight(int height) {
        this.height = height;
    }
    public int getTopMargin() {
        return topMargin;
    }
    public void setTopMargin(int topMargin) {
        this.topMargin = topMargin;
    }
    public int getBottomMargin() {
        return bottomMargin;
    }
    public void setBottomMargin(int bottomMargin) {
        this.bottomMargin = bottomMargin;
    }
    public void dispose() {
	composite.dispose();
    }
    public boolean isInitialized() {
	return initialized;
    }
    @Override public Iterator<Control> iterator() {
	return Arrays.asList(items).iterator();
    }
    private String getText(ILabelProvider provider, int row, int column) {
	CharSequence sequence = provider.getText(row, column);
	if (sequence == null) return null;
	return sequence.toString();
    }
    private void setBackground(Composite composite, Color color) {
	for (Control control: composite.getChildren())
	    control.setBackground(color);
    }
    private void setToolTipText(Composite composite, String string) {
	for (Control control: composite.getChildren())
	    control.setToolTipText(string);
    }
    private Control[] items;
    private Label separator;
    private int topMargin = 2;
    private int bottomMargin = 2;
    private TableView2 parent;
    private int height;
    private Composite composite;
    private boolean initialized = false;
    private boolean selected = false;
    private boolean isSeparator;
    private int index;
}
