/*
 *	Programmer:	Piotr Borek
 *	E-mail:		piotrborek@op.pl
 *	Copyright 2010  Piotr Borek
 *
 *	Distributed under the terms of the GPL (GNU Public License)
 *
 *   Jarre is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.widgets.views.table;

import jarre.widgets.common.views.IControlProvider;
import jarre.widgets.common.views.IEnabledProvider;
import jarre.widgets.common.views.ILabelProvider;
import jarre.widgets.common.views.ITableListener;
import jarre.widgets.common.views.ITableSelectionListener;
import jarre.widgets.layout.CustomGridData;
import jarre.widgets.layout.CustomGridLayout;
import jarre.widgets.utils.EventSlot;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.ScrollBar;

public class TableView2 extends Composite {
    public static final TableSeparator2 SEPARATOR = new TableSeparator2();
    private static final String COMPUTED_SIZE = "jarre.widgets.views.table.TableView2.size";
    public TableView2(Composite parent, int style) {
	super(parent, style);
	CustomGridLayout layout = new CustomGridLayout();
	layout.horizontalSpacing = 0;
	layout.marginHeight = 0;
	layout.marginWidth = 0;
	layout.verticalSpacing = 0;
	setLayout(layout);
	headerComposite = new Composite(this, SWT.NONE);
	headerComposite.setLayoutData(new CustomGridData(SWT.FILL, SWT.CENTER, true, false));
	headerComposite.setBackground(getDisplay().getSystemColor(SWT.COLOR_LIST_BACKGROUND));
	headerComposite.setVisible(false);
	final Composite composite = new Composite(this, SWT.NONE);
	composite.setLayoutData(new CustomGridData(SWT.FILL, SWT.FILL, true, true));
	main = new Composite(composite, SWT.NONE);
	main.setBackground(getDisplay().getSystemColor(SWT.COLOR_LIST_BACKGROUND));
	main.setLocation(0, 0);
	getDisplay().addFilter(SWT.MouseDown, mouseDownListener);
	getDisplay().addFilter(SWT.MouseDoubleClick, mouseDoubleClickListener);
	composite.addListener(SWT.Resize, new Listener() {
	    @Override public void handleEvent(Event event) {
		Rectangle area = composite.getClientArea();
		if (area.width == 0 || area.height == 0) return;
		Point size = new Point(area.width, area.height);
		mainSize = size;
		layoutTable();
	    }	    
	});
	final ScrollBar scrollBar = getVerticalBar();
	if (scrollBar != null)
	    scrollBar.addListener(SWT.Selection, new Listener() {
		@Override public void handleEvent(Event event) {
		    if (! scrollBar.isVisible()) return;
		    int value = scrollBar.getSelection();
		    if (lastValue < value) {
			if (main.getLocation().y != deltaY) {
			    indexError = -1;
			    layoutTable();
			    main.setLocation(0, deltaY);
			} else {
			    layoutTable();
			}
		    } else if (lastValue > value) {
			if (main.getLocation().y != 0) {
			    indexError = 0;
			    layoutTable();
			    main.setLocation(0, 0);
			} else {
			    layoutTable();
			}
		    }
		    lastValue = value;
		    TableRow2 newRow = findRow(selectionIndex);
		    if (newRow == null) {
			if (selection != null) selection.setSelected(false);
			selection = null;
		    } else {
			if (newRow != selection) {
			    if (selection != null) selection.setSelected(false);
			    newRow.setSelected(true);
			    selection = newRow;
			}
		    }
		}		
	    });
    }
    public void addSelectionListener(ITableSelectionListener listener) {
	selectionSlot.addListener(listener);
    }
    public void removeSelectionListener(ITableSelectionListener listener) {
	selectionSlot.removeListener(listener);
    }
    public void addDoubleClickListener(ITableListener listener) {
	doubleClickSlot.addListener(listener);
    }
    public void removeDoubleClickListener(ITableListener listener) {
	doubleClickSlot.removeListener(listener);
    }
    public IControlProvider getControlProvider() {
	return controlProvider;
    }
    public void setControlProvider(IControlProvider controlProvider) {
        this.controlProvider = controlProvider;
    }
    public ILabelProvider getLabelProvider() {
        return labelProvider;
    }
    public void setLabelProvider(ILabelProvider labelProvider) {
        this.labelProvider = labelProvider;
    }
    public IEnabledProvider getEnabledProvider() {
        return enabledProvider;
    }
    public void setEnabledProvider(IEnabledProvider enabledProvider) {
        this.enabledProvider = enabledProvider;
    }
    public void setHeaderVisible(boolean headerVisible) {
	this.headerVisible = headerVisible;
	headerComposite.setVisible(headerVisible);
    }
    public int getTableSize() {
	return rowList.size();
    }
    public TableColumn2[] getColumns() {
	return columnList.toArray(new TableColumn2[0]);
    }
    public int getSelectionIndex() {
	return selectionIndex;
    }
    public void setItemsCount(int count) {
	this.count = count;
	disposeRows();
	layoutTable();
    }
    public void updateData() {
	layoutTable();
    }
    TableRow2 findRow(Point coord) {
	TableRow2 hereIAm = null;
	for (TableRow2 row: rowList) {
	    if (row.getBounds().y < coord.y) hereIAm = row;
	    else break;
	}
	return hereIAm;
    }
    TableRow2 findRow(int index) {
	for (TableRow2 row: rowList)
	    if (row.getIndex() == index) return row;
	return null;
    }
    private void layoutTable() {
	computeRows();
	computeColumns();
	if (headerVisible) {
	    if (columnList.size() > 0) {
		int height = 0;
		for (TableColumn2 column: columnList) {
		    Point size = column.getColumnButton().computeSize(SWT.DEFAULT, SWT.DEFAULT);
		    if (height < size.y) height = size.y;
		}
		int x = 0;
		for (TableColumn2 column: columnList) {
		    int width = column.getWidth() + column.getLeftMargin() + column.getRightMargin();
		    column.getColumnButton().setBounds(
		        x,
		        0,
		        width,
		        height - 3
		    );
		    x += width;
		}
		headerComposite.setBounds(0, 0, x, height);
	    }
	}
	int y = 0;
	for (TableRow2 row: rowList) {
	    int x = 0; Control[] controls = row.getItems();
	    int height = row.getHeight() + row.getTopMargin() + row.getBottomMargin();
	    if (row.isSeparator()) {
		row.setBounds(2, y, mainSize.x - 4, height);
		y += height;
		continue;
	    } else { 
		row.setBounds(0, y, mainSize.x, height);
	    }
	    for (int i = 0; i < controls.length; i++) {
		Control control = controls[i];
		TableColumn2 column = columnList.get(i);
		int width = column.getWidth() + column.getLeftMargin() + column.getRightMargin();
		Point cSize = (Point) control.getData(COMPUTED_SIZE);
		int dy = (row.getHeight() - cSize.y) / 2;
		if (control instanceof Composite) dy = 0;
		control.setBounds(
		    x + column.getLeftMargin(),
		    row.getTopMargin() + dy,
		    column.getWidth(),
		    row.getHeight()
		);
		x += width;
	    }
	    y += height;
	}
	if (y < mainSize.y) main.setSize(mainSize);
	else main.setSize(mainSize.x, y);
    }
    private void computeRows() {
	int index = 0;
	ScrollBar scrollBar = getVerticalBar();
	if (scrollBar != null) index = scrollBar.getSelection() + indexError;
	if (index < 0) index = 0;
	int y = 0; int relIndex = -1;
	boolean scrollBarVisible = true;
	boolean recreate = false;
	if (firstOnTable != index) {
	    recreate = true;
	    firstOnTable = index;
	}
	for (TableRow2 row: rowList) row.setVisible(false);
	while (y < mainSize.y) {
	    relIndex++;
	    if (relIndex >= count) {
		scrollBarVisible = false;
		break;
	    }
	    if (index >= count) break;
	    TableRow2 row = null;
	    if (rowList.size() <= relIndex) {
		row = new TableRow2(this);
		rowList.add(row);
		row.initialize();
	    } else {
		row = rowList.get(relIndex);
	    }
	    row.setVisible(true);
	    row.setIndex(index++);
	    row.fillWidthData(recreate);
	    int maxHeight = 0;
	    if (row.isSeparator()) {
		Point cSize = row.computeSize(SWT.DEFAULT, SWT.DEFAULT);
		maxHeight = cSize.y;
	    } else {
		for (Control control: row.getItems()) {
		    Point cSize = control.computeSize(SWT.DEFAULT, SWT.DEFAULT);
		    control.setData(COMPUTED_SIZE, cSize);
		    if (maxHeight < cSize.y) maxHeight = cSize.y;
		}
	    }
	    row.setHeight(maxHeight);
	    y += maxHeight + row.getTopMargin() + row.getBottomMargin();;
	}
	deltaY = mainSize.y - y;
	if (! scrollBarVisible) {
	    main.setLocation(0, 0);
	    indexError = 0;
	}
	if (scrollBar != null) {
	    scrollBar.setVisible(scrollBarVisible);
	    scrollBar.setMinimum(0);
	    scrollBar.setThumb(1);
	    scrollBar.setIncrement(1);
	    scrollBar.setPageIncrement(1);
	    if (deltaY < 0) scrollBar.setMaximum(count - relIndex + 1);
	    else scrollBar.setMaximum(count - relIndex);
	}
    }
    private void computeColumns() { 
	int sum = 0;
	for (TableColumn2 column: columnList) {
	    column.getColumnButton().setVisible(headerVisible);
	    int max = 0;
	    for (Control control: column) {
		Point cSize = (Point) control.getData(COMPUTED_SIZE);
		if (cSize == null) continue;
		if (max < cSize.x) max = cSize.x;
	    }
	    column.setWidth(max);
	    sum += max + column.getLeftMargin() + column.getRightMargin();
	}
	int widthLeft = mainSize.x - sum;
	if (widthLeft > 0) {
	    ArrayList<TableColumn2> tmp = new ArrayList<TableColumn2>();
	    for (TableColumn2 column: columnList)
		if ((column.getAllignment() & SWT.FILL) == SWT.FILL) tmp.add(column);
	    if (! tmp.isEmpty()) {
		TableColumn2[] array = tmp.toArray(new TableColumn2[0]);
		while (widthLeft > 0) {
		    Arrays.sort(array, sortColumns);
		    TableColumn2 column = array[0];
		    column.setWidth(column.getWidth() + 1);
		    widthLeft--;
		}
	    }
	}
    }
    private void disposeRows() {
	for (TableRow2 r: rowList) r.dispose();
	rowList.clear();
	selection = null;
	selectionIndex = -1;
	firstOnTable = -1;
    }
    private Point getTableCoord(Event event) {
	if (isDisposed()) return null;
	Point coord = getDisplay().map((Control)event.widget, main, event.x, event.y);
	if (coord.x < 0) return null;
	if (coord.x > mainSize.x) return null;
	if (coord.y < 0) return null;
	if (coord.y > mainSize.y) return null;
	return coord;
    }
    ArrayList<TableColumn2> columnList = new ArrayList<TableColumn2>();
    ArrayList<TableRow2> rowList = new ArrayList<TableRow2>();
    TableRow2 selection = null;
    int selectionIndex = -1;
    EventSlot<ITableSelectionListener, Integer> selectionSlot = new EventSlot<ITableSelectionListener, Integer>() {
	@Override public void handleEvent(ITableSelectionListener listener, Integer arg) {
	    listener.handleEvent(TableView2.this, selection.getItems(), selectionIndex, arg);
	}
    };
    EventSlot<ITableListener, Object> doubleClickSlot = new EventSlot<ITableListener, Object>() {
	@Override public void handleEvent(ITableListener listener, Object arg) {
	    listener.handleEvent(TableView2.this);
	}	
    };
    Composite main;
    Composite headerComposite;
    private int firstOnTable = -1;
    private int count;
    private int deltaY;
    private int indexError;
    private int lastValue = 0;
    private Point mainSize = new Point(10, 10);
    private IControlProvider controlProvider;
    private ILabelProvider labelProvider;
    private IEnabledProvider enabledProvider;
    private boolean headerVisible = false;
    private Listener mouseDownListener = new Listener() {
	@Override public void handleEvent(Event event) {
	    Point coord = getTableCoord(event);
	    if (coord == null) return;
	    TableRow2 hereIAm = findRow(coord);
	    if (hereIAm == null) return;
	    if (hereIAm.isSeparator()) return;
	    if (selection != null) selection.setSelected(false);
	    selectionIndex = hereIAm.getIndex();
	    selection = hereIAm;
	    selection.setSelected(true);
	}	
    };
    private Listener mouseDoubleClickListener = new Listener() {
	@Override public void handleEvent(Event event) {
	    Point coord = getTableCoord(event);
	    if (coord == null) return;
	}
    };
    private Comparator<TableColumn2> sortColumns = new Comparator<TableColumn2>() {
	@Override public int compare(TableColumn2 a, TableColumn2 b) {
	    return a.getWidth() - b.getWidth();
	}	
    };
}
