/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.widgets.windows;

import jarre.widgets.Composite2;
import jarre.widgets.common.App;
import jarre.widgets.utils.Animation;

import java.awt.SplashScreen;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;

public abstract class AppWindow {
    private Shell shell;
    private int returnCode = SWT.NONE;
    private Composite2 composite;
    private boolean fullScreen = false;
    private Animation animation;
    public AppWindow(Shell shell, String title) {
	this(shell, title, App.NONE);
    }
    public AppWindow(final Shell shell, String title, int flags) {
	this.shell = shell;
	shell.setText(title);
	shell.setLayout(new FillLayout());
	shell.addListener(SWT.Close, new Listener() {
	    @Override public void handleEvent(Event event) {
		animation.dispose();
		AppWindow.this.closeShell(event);
	    } 
	});
	if ((flags & App.CREATE_MENU_BAR) == App.CREATE_MENU_BAR) {
	    Menu bar = new Menu(shell, SWT.BAR);
	    createMenuBar(bar);
	    shell.setMenuBar(bar);
	}
	animation = new Animation(shell.getDisplay(), 48, 48, "data/process-working.png", shell.getDisplay().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND));
	composite = new Composite2(shell, SWT.NONE);
	createControl(composite.getComposite());
    }
    public int open() {
	Display display = shell.getDisplay();
	shell.open();
	display.timerExec(1000, new Runnable() {
	    @Override public void run() {
		SplashScreen splash = SplashScreen.getSplashScreen();
		if (splash != null) splash.close();
	    }
	});
	while (!shell.isDisposed()) {
	    if (!display.readAndDispatch()) display.sleep();
	}
	return returnCode;
    }
    public void close(int returnCode) {
	this.returnCode = returnCode;
	shell.close();
    }
    public static void readAndDispatch(Shell shell) {
	while (! shell.isDisposed()) {
	    if (! shell.getDisplay().readAndDispatch()) break;
	}
    }
    public Shell getShell() {
	return shell;
    }
    public Display getDisplay() {
	return shell.getDisplay();
    }
    public void setFullscreen(boolean fullScreen) {
	this.fullScreen = fullScreen;
	shell.setFullScreen(fullScreen);
    }
    public boolean isFullscreen() {
	return fullScreen;
    }
    public void execTask(Runnable runnable) {
	AppWindow.readAndDispatch(shell);
	composite.doTask(runnable);
    }
    public void syncExec(Runnable runnable) {
	getDisplay().syncExec(runnable);
    }
    public void asyncExec(Runnable runnable) {
	getDisplay().asyncExec(runnable);
    }
    protected void createMenuBar(Menu menu) {
    }
    protected void closeShell(Event event) {
    }
    protected abstract void createControl(Composite parent);
}
